// 003 u_c_bridge_drawings
// 18 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- MUST NOT use Canvas.Rectangle etc

(*$r+*)

unit u_c_bridge_drawings;
  interface
    uses Graphics
        , u_c_basic_object
        // should use NONE of this here: u_c_control  // get access to the true canvas
        , u_c_bridge_canvas
        ;

    type // -- here are the drawing kinds we need

         // -- in the ancestor,
         // --  place ALL the functions that will ever be needed (not all will be used
         // --  by each descendent drawin)
         c_abstract_drawing= class(c_basic_object)
                              m_c_canvas_ref: c_abstract_canvas;
                              m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height: Integer;

                              m_on_mouse_down, m_on_mouse_move, m_on_mouse_up: t_on_drawing_mouse_down;

                              Constructor create_abstract_drawing(p_name: String;
                                  p_c_canvas_ref: c_abstract_canvas;
                                  p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
                              // Open Close SetOrigin Iconify
                              // -- draw text, line, poly, rect ...
                              // function f_c_get_drawing_implementation: c_abstract_canvas;

                              function f_display_drawing: String; Virtual;

                              procedure save_graphic_context;
                              procedure restore_graphic_context;
                              procedure set_pen_color(p_color: Integer);
                              procedure set_pen_mode(p_pen_mode: tPenMode);
                              procedure set_brush_style(p_brush_style: tBrushStyle);
                              procedure set_brush_color(p_brush_color: Integer);
                              procedure set_font_height(p_font_height: Integer);
                              function f_get_font_height: Integer;

                              function f_character_width(p_character: Char): Integer;
                              function f_character_height(p_character: Char): Integer;

                              procedure draw_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer);
                              procedure draw_triangle(p_left, p_top, p_width, p_height: Integer);
                              procedure draw_rectangle(p_left, p_top, p_width, p_height: Integer);
                              procedure draw_text(p_x, p_y: Integer; p_text: String); Virtual;
                              procedure shrink_drawing(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer); Virtual;

                              procedure fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);
                              procedure draw_bitmap(p_x, p_y: Integer;  p_c_bitmap: tBitmap);

                              procedure draw_content; Virtual; Abstract;
                            end; // c_abstract_drawing

(*
         c_button_drawing= class(c_abstract_drawing)
                            // m_x, m_y, m_width, m_height: Integer;

                            Constructor create_button_drawing(p_name: String;
                                p_c_canvas_ref: c_abstract_canvas;
                                p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
                          end; // c_dialog_drawing
*)
         c_label_drawing= class(c_abstract_drawing)
                            // m_x, m_y, m_width, m_height: Integer;

                            Constructor create_label_drawing(p_name: String;
                                p_c_canvas_ref: c_abstract_canvas;
                                p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
                            procedure set_caption(p_caption: String);
                          end; // c_dialog_drawing
{
         c_memo_drawing= class(c_abstract_drawing)
                          Constructor create_memo_drawing(p_name: String;
                              p_c_canvas_ref: c_abstract_canvas;
                              p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
                        end; // c_dialog_drawing
         // c_dialog_drawing
}
         c_client_region_drawing= Class(c_abstract_drawing)
                                    Constructor create_client_region_drawing(p_name: String;
                                        p_c_canvas_ref: c_abstract_canvas;
                                        p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
                                    procedure draw_content; Override;
                                  end; // c_client_region_drawing

         c_scrollbar_drawing= class(c_abstract_drawing)
                                Constructor create_scrollbar_drawing(p_name: String;
                                   p_c_canvas_ref: c_abstract_canvas;
                                   p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
                                procedure draw_content; Override;
                                procedure shrink_drawing(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer); Override;
                                procedure set_position(p_position: Integer);
                                function f_get_position: Integer;
                              end; // c_scrollbar_drawing

  implementation
    uses SysUtils, u_c_display
        , u_graphic
        ;

    // -- c_abstract_drawing

    Constructor c_abstract_drawing.create_abstract_drawing(p_name: String;
        p_c_canvas_ref: c_abstract_canvas;
        p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
      begin
        Inherited create_basic_object(p_name);
        m_c_canvas_ref:= p_c_canvas_ref;

        m_drawing_left:= p_drawing_left; m_drawing_top:= p_drawing_top;
        m_drawing_width:= p_drawing_width; m_drawing_height:= p_drawing_height;
      end; // create_abstract_drawing

    function c_abstract_drawing.f_display_drawing: String;
      begin
        Result:= m_name+ f_display_wh_rectangle(m_drawing_left, m_drawing_top,
            m_drawing_width, m_drawing_height);
      end; // f_display_drawing

    procedure c_abstract_drawing.save_graphic_context;
      begin
        m_c_canvas_ref.save_device_context;
      end; // save_graphic_context

    procedure c_abstract_drawing.restore_graphic_context;
      begin
        m_c_canvas_ref.restore_device_context;
      end; // restore_graphic_context

    procedure c_abstract_drawing.set_pen_color(p_color: Integer);
      begin
        m_c_canvas_ref.set_pen_color(p_color);
      end; // set_pen_color

    procedure c_abstract_drawing.set_pen_mode(p_pen_mode: tPenMode);
      begin
        m_c_canvas_ref.set_pen_mode(p_pen_mode);
      end; // set_pen_mode

    procedure c_abstract_drawing.set_brush_style(p_brush_style: tBrushStyle);
      begin
        m_c_canvas_ref.set_brush_style(p_brush_style);
      end; // set_brush_style

    procedure c_abstract_drawing.set_brush_color(p_brush_color: Integer);
      begin
        m_c_canvas_ref.set_brush_color(p_brush_color);
      end; // set_brush_color

    procedure c_abstract_drawing.set_font_height(p_font_height: Integer);
      begin
        if p_font_height< 0
          then display_bug_halt('neg_font');
        m_c_canvas_ref.canvas_set_font_height(p_font_height);
      end; // set_font_height

    function c_abstract_drawing.f_get_font_height: Integer;
      begin
        Result:= m_c_canvas_ref.f_canvas_get_font_height;
        if Result< 0
          then display_bug_halt('f_neg_font');
      end; // f_get_font_height

    function c_abstract_drawing.f_character_width(p_character: Char): Integer;
      begin
        Result:= m_c_canvas_ref.f_canvas_character_width(p_character);
      end; // f_character_width

    function c_abstract_drawing.f_character_height(p_character: Char): Integer;
      begin
        Result:= m_c_canvas_ref.f_canvas_character_height(p_character);
      end; // f_character_height

    procedure c_abstract_drawing.draw_text(p_x, p_y: Integer; p_text: String);
      begin
        with m_c_canvas_ref do
          // device_text_out(m_drawing_left+ p_x, m_drawing_top+ p_y, p_text);
          device_text_out(p_x, p_y, p_text);
      end; // draw_text

    procedure c_abstract_drawing.draw_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer);
      begin
        with m_c_canvas_ref do
          canvas_line(p_x_1, p_y_1, p_x_2, p_y_2);
      end; // draw_line

    procedure c_abstract_drawing.draw_triangle(p_left, p_top, p_width, p_height: Integer);
        // -- use the device primitives to draw the shape
      begin
        if m_c_canvas_ref= Nil
          then display_bug_stop('imp_nil');

        with m_c_canvas_ref do
        begin
          display(f_display_wh_rectangle(p_left, p_top, 99, 99));
(*
          if m_c_canvas= Nil
            then display_bug_stop('canvas_nil');
*)
          canvas_line(p_left+ p_width div 2, p_top, p_left+ p_width, p_top);
          canvas_line(p_left+ p_width, p_top, p_left+ p_width, p_top+ p_height);
          canvas_line(p_left+ p_width, p_top+ p_height, p_left, p_top+ p_height);
          canvas_line(p_left, p_top+ p_height, p_left+ p_width div 2, p_top);
        end; // m_c_canvas
      end; // draw_triangle

    procedure c_abstract_drawing.draw_rectangle(p_left, p_top, p_width, p_height: Integer);
      begin
        m_c_canvas_ref.draw_rectangle(p_left, p_top, p_width, p_height);
        // stop(m_name);
      end; // draw_rectangle

    procedure c_abstract_drawing.shrink_drawing(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer);
      begin
        display('> shrink_drawing '+ f_display_wh_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height));

        Inc(m_drawing_left, p_delta_left);
        Inc(m_drawing_top, p_delta_top);
        Dec(m_drawing_width, p_delta_left+ p_delta_right);
        Dec(m_drawing_height, p_delta_top+ p_delta_bottom);

        display('< shrink_drawing '+ f_display_wh_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height));
      end; // shrink_drawing

    procedure c_abstract_drawing.fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);
      begin
        m_c_canvas_ref.fill_rectangle(p_left, p_top, p_width, p_height, p_color);
      end; // fill_rectangle

    procedure c_abstract_drawing.draw_bitmap(p_x, p_y: Integer; p_c_bitmap: tBitmap);
      begin
        m_c_canvas_ref.canvas_draw_bitmap(p_x, p_y, p_c_bitmap);
      end; // draw_bitmap

    // -- c_label_drawing

    Constructor c_label_drawing.create_label_drawing(p_name: String;
        p_c_canvas_ref: c_abstract_canvas;
        p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
      begin
      end; // create_label_drawing

    procedure c_label_drawing.set_caption(p_caption: String);
      begin
      end; // c_dialog_drawing

(*
    // -- c_button_drawing

    Constructor c_button_drawing.create_button_drawing(p_name: String;
        p_c_canvas_ref: c_abstract_canvas;
        p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
      begin
        Inherited create_abstract_drawing(p_name, p_c_canvas_ref,
            p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height);
      end; // create_button_drawing
*)

{
    // -- c_memo_drawing

    Constructor c_memo_drawing.create_memo_drawing(p_name: String;
        p_c_canvas_ref: c_abstract_canvas;
        p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
      begin
        Inherited create_abstract_drawing(p_name, p_c_canvas_ref,
            p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height);
      end; // create_memo_drawing
}
    // -- c_client_region_drawing

    Constructor c_client_region_drawing.create_client_region_drawing(p_name: String;
        p_c_canvas_ref: c_abstract_canvas;
        p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
      begin
        Inherited create_abstract_drawing(p_name, p_c_canvas_ref,
            p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height);

        draw_content;
      end; // create_client_region_drawing

    procedure c_client_region_drawing.draw_content;
      begin
        display('> draw_content '+ f_display_drawing);
        save_graphic_context;
        m_c_canvas_ref.set_pen_color(clFuchsia);
        // display('height '+ IntToStr(p_height));
        // -- if wants to see the scrollbar control, must only use the ouline
        draw_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height);
        // stop('content');
        restore_graphic_context;
        display('< draw_content '+ m_name);
      end; // draw_content

    // -- c_scrollbar_drawing

    Constructor c_scrollbar_drawing.create_scrollbar_drawing(p_name: String;
        p_c_canvas_ref: c_abstract_canvas;
        p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height: Integer);
      begin
        Inherited create_abstract_drawing(p_name, p_c_canvas_ref,
            p_drawing_left, p_drawing_top, p_drawing_width, p_drawing_height);

(*
        draw_rectangle(p_left, p_top, p_width, 15);
        draw_rectangle(p_left, p_top+ 15+ 2, p_width, p_height- 15- 2);
*)
      end; // create_scrollbar_drawing

    procedure c_scrollbar_drawing.draw_content;
      begin
        display('> draw_content '+ f_display_drawing);
        save_graphic_context;
        m_c_canvas_ref.set_pen_color(clBlue);
        draw_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height);
        // stop('scroll_draw');
        restore_graphic_context;
        display('< draw_content '+ m_name);
      end; // draw_content

    procedure c_scrollbar_drawing.shrink_drawing(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer);
        // -- the right scroll-bar is some kind of "fat right edge"
        // --   => if moves the right edge, move the scroll bar
      // var l_left, l_right: Integer
      begin
        display('> shrink_scroll_drawing '+ f_display_wh_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height)
          + ' '+ f_display_wh_rectangle(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom));

        // l_right:= m_drawing_left- p_delta_right;
        // l_left:= p_delta_left-
        Dec(m_drawing_left, - p_delta_right);

        Inc(m_drawing_top, p_delta_top);
        // Dec(m_drawing_width, p_delta_left+ p_delta_right);
        Dec(m_drawing_height, (p_delta_top- p_delta_bottom) );

        display('< shrink_scroll_drawing '+ f_display_wh_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height));
      end; // shrink_drawing

    procedure c_scrollbar_drawing.set_position(p_position: Integer);
      begin
      end; // set_position

    function c_scrollbar_drawing.f_get_position: Integer;
      begin
      end; // f_get_position
      
    begin // u_c_abstract_drawing
    end. // u_c_abstract_drawing
