// 002 u_c_command_kinds
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_command_kinds;
  interface
    uses u_c_command
        , u_c_document
        ;

    type c_insert_command= Class(c_abstract_command)
                             m_c_document_ref: c_document;
                             // -- m_name contains the name of the character (debug)
                             // -- this is the insertion point
                             m_row_index, m_column_index: Integer;

                             Constructor create_insert_command(p_name: String;
                                 p_c_document_ref: c_document);
                             procedure execute_command; Override;
                             procedure undo_command; Override;
                           end; // c_command

         c_delete_command= Class(c_abstract_command)
                             Constructor create_delete_command(p_name: String;
                                 p_font_name: String);
                             procedure execute_command; Override;
                           end; // c_command

         // -- set the font of the selected text
         c_font_command= class(c_abstract_command)
                           m_font_name: String;
                           Constructor create_font_command(p_name: String;
                               p_font_name: String);
                           procedure execute_command; Override;
                         end; // c_font_command
         // -- c_save_command
         // -- c_quit command

  implementation
    uses SysUtils
        , u_c_display
        , u_c_row_glyph
        ;

    // -- c_insert_command

    Constructor c_insert_command.create_insert_command(p_name: String;
        p_c_document_ref: c_document);
      begin
        // display('create_insert_command '+ p_name[1]);
        Inherited create_basic_object(p_name);
        // display('char '+ m_name[1]+ '|'+ m_name+ '<');
        m_c_document_ref:= p_c_document_ref;
      end; // create_insert_command

    procedure c_insert_command.execute_command;
      var l_row_index, lcolumn_index: Integer;
      begin
        with m_c_document_ref do
        begin
          f_c_insert_leaf_glyph(Self.m_name[1]);

          // -- column_index-1 since by now the caret has been moved ahead
          m_row_index:= m_c_caret.m_caret_row_index;
          m_column_index:= m_c_caret.m_caret_element_index- 1;
        end; // with m_c_document_ref
      end; // execute_command

    procedure c_insert_command.undo_command;
      var l_c_row_glyph: c_row_glyph;
      begin
        display('> undo_command');
        with m_c_document_ref do
        begin
          l_c_row_glyph:= m_c_root_page_glyph.f_c_glyph(m_row_index) as c_row_glyph;
          l_c_row_glyph.remove_glyph_by_index(m_column_index);

          m_c_root_page_glyph.compute_placement;
          clear_page;
          draw_page_and_caret;
        end; // with m_c_document_ref
        display('< undo_command');
      end; // undo_command

    // -- c_delete_command

    Constructor c_delete_command.create_delete_command(p_name: String;
        p_font_name: String);
      begin
      end; // create_delete_command

    procedure c_delete_command.execute_command;
      begin
      end; // execute_command

    // -- c_font_command

    Constructor c_font_command.create_font_command(p_name: String;
        p_font_name: String);
      begin
        Inherited create_basic_object(p_name);
        m_font_name:= p_font_name;
      end; // create_font_command

    procedure c_font_command.execute_command;
      begin
      end; // execute_command
(*

    procedure c_font_command.change_font;
      begin
      end; // change_font
*)
    begin // u_c_font_command
    end. // u_c_font_command
