// 002 u_c_glyph
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- choices

(*$r+*)

unit u_c_glyph;
  interface
    uses Windows, Classes, ExtCtrls, Graphics
        , u_c_basic_object
        , u_c_bridge_drawings
        , u_c_visitor
        ;

    const // -- if starts at 0, will not be able to select
          k_x_margin= 2; k_y_margin= 2;
          k_caret_width= 3;

    type // -- +- "abstract for drawing"
         c_glyph= class(c_basic_object)
                    _m_c_parent_glyph: c_glyph;
                    m_glyph_id: Integer;
                    // -- m_x, m_y are RELATIVE to the parent
                    m_x, m_y, m_width, m_height: Integer;

                    m_line_color, m_brush_color: Integer;
                    m_is_selected: Boolean;

                    // -- the size for reorganizing (m_with and m_height are
                    // --   the character enclosing rectangle)
                    // -- must be this high because decorator uses c_glyph as ancestor
                    m_width_max, m_height_max: Integer;

                    Constructor create_glyph(p_name: String;
                        p_c_parent_glyph: c_glyph);

                    function f_c_parent_glyph: c_glyph;
                    function f_c_self: c_glyph;

                    function f_display_glyph: String; Virtual;
                    procedure display_glyph; Virtual;

                    function f_absolute_x: Integer;
                    function f_absolute_y: Integer; Virtual;

                    function f_line_color: Integer;
                    function f_brush_color: Integer;

                    procedure draw(p_c_client_region_drawing: c_client_region_drawing); Virtual; Abstract;

                    function f_bounding_rectangle: tRect; Virtual; Abstract;
                    function f_point_in_glyph(p_point: tPoint): Boolean; Virtual; Abstract;
                    function f_glyph_in_rectangle(p_absolute_rectangle: tRect): Boolean; Virtual;
                    function f_c_glyph_of_point(p_point: tPoint): c_glyph; Virtual;
                    procedure select_in_rectangle(p_absolute_rectangle: tRect); Virtual;

                    procedure compute_placement; Virtual;
                    procedure change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing); Virtual;

                    function f_c_create_iterator: tObject; Virtual;

                    procedure accept_visitor(p_c_visitor: c_abstract_visitor); Virtual;

                    Destructor Destroy; Override;
                  end; // c_glyph

         c_composite_glyph= class(c_glyph)
                              _m_c_glyph_list: tList;

                              Constructor create_composite_glyph(p_name: String;
                                  p_c_parent_glyph: c_glyph);

                              function f_c_self: c_composite_glyph;
                              function f_glyph_count: Integer;
                              function f_c_glyph(p_glyph_index: Integer): c_glyph;
                              function f_index_of(p_c_glyph: c_glyph): Integer;

                              procedure append_glyph(p_c_glyph: c_glyph); Virtual;
                              procedure insert_glyph(p_insert_index: Integer; p_c_glyph: c_glyph); Virtual;
                              procedure remove_glyph_by_index(p_glyph_index: Integer);
                              procedure remove_glyph(p_c_glyph: c_glyph);

                              procedure display_glyph; Override;

                              procedure draw(p_c_client_region_drawing: c_client_region_drawing); Override;
                              function f_bounding_rectangle: tRect; Override;
                              function f_point_in_glyph(p_point: tPoint): Boolean; Override;
                              function f_c_glyph_of_point(p_point: tPoint): c_glyph; Override;
                              procedure select_in_rectangle(p_absolute_rectangle: tRect); Override;

                              procedure compute_placement; Override;
                              procedure change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing); Override;

                              procedure shrink(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer); Virtual;

                              function f_c_create_iterator: tObject; Override;

                              procedure nil_structure;
                              procedure free_structure;

                              Destructor Destroy; Override;
                            end; // c_composite_glyph

  implementation
    uses Math, SysUtils, u_c_display
        , u_strings
        , u_c_iterator_kinds
        ;

    // -- c_glyph

    var s_glyph_id: Integer= 1;

    Constructor c_glyph.create_glyph(p_name: String;
        p_c_parent_glyph: c_glyph);
      begin
        Inherited create_basic_object(p_name);
        _m_c_parent_glyph:= p_c_parent_glyph;
        m_glyph_id:= s_glyph_id;
        Inc(s_glyph_id);
      end; // create_glyph

    function c_glyph.f_c_parent_glyph: c_glyph;
      begin
        Result:= _m_c_parent_glyph;
      end; // f_c_parent_glyph

    function c_glyph.f_c_self: c_glyph;
      begin
        Result:= Self;
      end; // f_c_self

    function c_glyph.f_display_glyph: String;
      var l_selected: String;
      begin
        if m_is_selected
          then l_selected:= ' sel'
          else l_selected:= '    ';
        Result:= m_name+ Format('%3d %3d   %3d %3d  %3d %3d',
            [m_x, m_y, m_width, m_height, m_width_max, m_height_max])+ l_selected;
      end; // f_display_glyph

    procedure c_glyph.display_glyph;
      begin
        display(f_display_glyph);
      end; // display_glyph
      
    function c_glyph.f_line_color: Integer;
      begin
        Result:= clBlack;
      end; // f_line_color

    function c_glyph.f_brush_color: Integer;
      begin
        if m_is_selected
          then Result:= clLime
          else Result:= clWhite;
      end; // f_brush_color

    function c_glyph.f_absolute_x: Integer;
      begin
        if f_c_parent_glyph= Nil
          then Result:= m_x
          else Result:= f_c_parent_glyph.m_x+ m_x;
      end; // f_absolute_x

    function c_glyph.f_absolute_y: Integer;
      begin
        if f_c_parent_glyph= Nil
          then Result:= m_y
          else Result:= f_c_parent_glyph.m_y+ m_y;
      end; // f_absolute_y

    function c_glyph.f_glyph_in_rectangle(p_absolute_rectangle: tRect): Boolean;
      begin
        with p_absolute_rectangle do
          Result:= (f_absolute_x>= Left) and (f_absolute_x+ m_width<= Right)
              and (f_absolute_y>= Top) and (f_absolute_y+ m_height<= Bottom);
        display(f_display_glyph+ ' '+ f_display_TF(Result));
      end; // f_glyph_in_rectangle

    function c_glyph.f_c_glyph_of_point(p_point: tPoint): c_glyph;
      var l_absolute_x, l_absolute_y: Integer;
      begin
        l_absolute_x:= f_absolute_x;
        l_absolute_y:= f_absolute_y;

        if (l_absolute_x<= p_point.X) and (l_absolute_x+ m_width>= p_point.X)
           and (l_absolute_y<= p_point.Y) and (l_absolute_y+ m_height>= p_point.Y)
          then Result:= Self
          else Result:= Nil;
      end; // f_c_glyph_of_point

    procedure c_glyph.select_in_rectangle(p_absolute_rectangle: tRect);
        // -- toggle the selected boolean
      begin
        m_is_selected:= f_glyph_in_rectangle(p_absolute_rectangle);
      end; // select_in_rectangle

    procedure c_glyph.compute_placement;
      begin
        // -- nothing to do
        // -- avoid abstract warnings
      end; // compute_placement

    procedure c_glyph.change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing);
      begin
        // -- avoid warnings. Only used in character descendent
        display('no_font '+ m_name);
      end; // change_selected_character_font

    function c_glyph.f_c_create_iterator: tObject;
      begin
        Result:= c_null_iterator.create_null_iterator('Null_iter', Self);
      end; // f_c_create_iterator

    procedure c_glyph.accept_visitor(p_c_visitor: c_abstract_visitor);
      begin
        // -- avoid problems for glyphs which did not override accept_visitor
        // display('in_accept');
      end; // accept_visitor
      
    Destructor c_glyph.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_composite_glyph

    Constructor c_composite_glyph.create_composite_glyph(p_name: string;
        p_c_parent_glyph: c_glyph);
      begin
        Inherited create_glyph(p_name, p_c_parent_glyph);

        _m_c_glyph_list:= tList.Create;
      end; // create_composite_glyph

    function c_composite_glyph.f_c_self: c_composite_glyph;
      begin
        Result:= Self;
      end; // f_c_self

    function c_composite_glyph.f_glyph_count: integer;
      begin
        Result:= _m_c_glyph_list.Count;
      end; // f_glyph_count

    function c_composite_glyph.f_c_glyph(p_glyph_index: integer): c_glyph;
      begin
        Result:= c_glyph(_m_c_glyph_list[p_glyph_index]);
      end; // f_c_glyph

    function c_composite_glyph.f_index_of(p_c_glyph: c_glyph): Integer;
      begin
        Result:= _m_c_glyph_list.IndexOf(p_c_glyph);
      end; // f_index_of

    procedure c_composite_glyph.append_glyph(p_c_glyph: c_glyph);
      begin
        _m_c_glyph_list.Add(p_c_glyph);
      end; // append_glyph

    procedure c_composite_glyph.insert_glyph(p_insert_index: Integer; p_c_glyph: c_glyph);
      begin
        _m_c_glyph_list.Insert(p_insert_index, p_c_glyph);
      end; // insert_glyph

    procedure c_composite_glyph.remove_glyph_by_index(p_glyph_index: Integer);
      begin
        // -- TODO: should also FREE ?
        f_c_glyph(p_glyph_index).Free;
        _m_c_glyph_list.Delete(p_glyph_index);
      end; // remove_glyph

    procedure c_composite_glyph.remove_glyph(p_c_glyph: c_glyph);
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_c_glyph);
        if l_index_of>= 0
          then // -- TODO call previous
               // -- TODO: should also FREE ?
               _m_c_glyph_list.Delete(l_index_of);
      end; // remove_glyph

    procedure c_composite_glyph.display_glyph;
      var l_glyph_index: Integer;
      begin
        // Exit;
        display(IntToHex(Integer(Self), 4)+ ' '+ f_display_glyph);

        change_indentation(2);
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_glyph_index) do
          begin
            display(m_name);
(*
            if f_c_self is c_composite_glyph
              then
                with f_c_self as c_composite_glyph do
                // ?? abstract error 
                  // display_glyph;
*)
          end;

        change_indentation(-2);
      end; // display_glyph

    procedure c_composite_glyph.draw(p_c_client_region_drawing: c_client_region_drawing);
      var l_glyph_index: Integer;
      begin
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          f_c_glyph(l_glyph_index).draw(p_c_client_region_drawing);
      end; // draw

    function c_composite_glyph.f_bounding_rectangle: trect;
        // -- compute and return the enclosing rectangle of all the children
      var l_glyph_index: Integer;
      begin
        Result:= Rect(0, 0, 0, 0);

        for l_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_glyph_index) do
          begin
            Result.Left:= Min(Result.Left, m_x);
            Result.Top:= Min(Result.Top, m_y);
            Result.Right:= Max(Result.Right, m_x+ m_width);
            Result.Bottom:= Min(Result.Bottom, m_y+ m_height);
          end; // for l_glyph_index, with f_c_glyph

        // -- todo: also compute / or set the composite bounding rectangle ?
      end; // f_bounding_rectangle

    function c_composite_glyph.f_point_in_glyph(p_point: tpoint): boolean;
      var l_glyph_index: Integer;
      begin
        Result:= False;
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_glyph_index) do
            if f_point_in_glyph(p_point)
              then begin
                  Result:= True;
                  Break;
                end;
      end; // f_point_in_glyph

    function c_composite_glyph.f_c_glyph_of_point(p_point: tPoint): c_glyph;
      var l_glyph_index: Integer;
      begin
        Result:= Nil;

        for l_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_glyph_index) do
          begin
            Result:= f_c_glyph_of_point(p_point);
            if Result<> Nil
              Then Break;
          end;
      end; // f_c_glyph_of_point

    procedure c_composite_glyph.select_in_rectangle(p_absolute_rectangle: tRect);
      var l_glyph_index: Integer;
      begin
        Inherited;

        // -- also compute children's selection
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          f_c_glyph(l_glyph_index).select_in_rectangle(p_absolute_rectangle);
      end; // select_in_rectangle

    procedure c_composite_glyph.compute_placement;
      // var l_glyph_index: Integer;
      begin
(*
        display('> composite.compute_composite_placement_recursive '+ m_name);

        display('= composite.compute_placement_before_virt');
        // compute_placement_before;

        // -- find each row's height
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_glyph_index) do
            if f_c_self is c_composite_glyph
              then
                with c_composite_glyph(f_c_self) do
                  compute_placement;

        // -- for a row descendent with compute horizontally
        display('= composite.compute_placement_after_virt');
        // compute_placement_after;
        display('< composite.compute_composite_placement_recursive ');
*)
      end; // compute_placement

    procedure c_composite_glyph.change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing);
      var l_glyph_index: Integer;
      begin
        // -- find each row's height
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_glyph_index) do
            change_selected_character_font(p_c_client_region_drawing);
      end; // change_selected_character_size

    procedure c_composite_glyph.shrink(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer);
      begin
        Inc(m_x, p_delta_left);
        Inc(m_y, p_delta_top);
        Inc(m_width_max, p_delta_right- p_delta_left);
        Inc(m_height_max, p_delta_bottom- p_delta_top);
      end; // shrink

    function c_composite_glyph.f_c_create_iterator: tObject;
      begin
        Result:= c_leaf_iterator.create_leaf_iterator('iter_', Self);
      end; // f_c_create_iterator

    procedure c_composite_glyph.nil_structure;
      var l_glyph_index: Integer;
      begin
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          _m_c_glyph_list[l_glyph_index]:= Nil;
      end; // nil_structure

    procedure c_composite_glyph.free_structure;
      var l_glyph_index: Integer;
      begin
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          f_c_glyph(l_glyph_index).Free;
        _m_c_glyph_list.Free;
      end; // free_structure

    Destructor c_composite_glyph.Destroy;
      var l_glyph_index: Integer;
      begin
        for l_glyph_index:= 0 to f_glyph_count- 1 do
          f_c_glyph(l_glyph_index).Free;
        _m_c_glyph_list.Free;

        Inherited;
      end; // Destroy

    begin // u_tPaintBox
    end. // u_tPaintBox




