// 002 u_c_glyph_kinds
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_glyph_kinds;
  interface
    uses Windows, ExtCtrls, Graphics, u_c_glyph
        , u_c_bridge_drawings
        , u_c_visitor
        ;

    const k_rectangle_width_fat= 4;
    type // -- a parent for all the leaf kinds
         // --   to be able to use IS
         c_leaf_glyph= Class(c_glyph)
                       end;

         c_character_glyph= class(c_leaf_glyph)
                              m_font_height: Integer;

                              Constructor create_character_glyph(p_name: String;
                                  p_c_parent_glyph: c_glyph; p_width, p_height: Integer);

                              procedure draw(p_c_client_region_drawing: c_client_region_drawing); Override;
                              procedure change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing); Override;
                              procedure accept_visitor(p_c_visitor: c_abstract_visitor); Override;
                            end; // c_character_glyph

         c_rectangle_glyph= class(c_leaf_glyph)
                              m_font_height: Integer;

                              Constructor create_rectangle_glyph(p_name: String;
                                  p_c_parent_glyph: c_glyph; p_width, p_height: Integer);

                              procedure change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing); Override;
                              procedure draw(p_c_client_region_drawing: c_client_region_drawing); Override;
                            end; // c_rectangle_glyph

         c_bitmap_glyph= Class(c_leaf_glyph)
                           m_c_bitmap: tBitmap;

                           Constructor create_bitmap_glyph(p_name: String;
                                p_c_parent_glyph: c_glyph; p_width, p_height: Integer;
                                p_bitmap_name: String);

                            procedure change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing); Override;
                            procedure draw(p_c_client_region_drawing: c_client_region_drawing); Override;
                            Destructor Destroy; Override;
                         end; // c_bitmap_glyph

  implementation
    uses SysUtils, u_c_display, u_graphic;

    // -- c_character_glyph

    Constructor c_character_glyph.create_character_glyph(p_name: string;
        p_c_parent_glyph: c_glyph;  p_width, p_height: Integer);
      begin
        if Length(p_name)= 0
          then p_name:= '?';
        Inherited create_glyph(p_name, p_c_parent_glyph);

        m_width:= p_width; m_height:= p_height;
      end; // create_character_glyph

    procedure c_character_glyph.draw(p_c_client_region_drawing: c_client_region_drawing);
      var l_x, l_y: Integer;
          l_font_height, l_width: Integer;
      begin
        l_x:= f_c_parent_glyph.m_x+ m_x;
        l_y:= f_c_parent_glyph.m_y+ m_y;

        if m_is_selected or (m_font_height<> 0)
          then
            with p_c_client_region_drawing do
            begin
              save_graphic_context;

              if Self.m_is_selected
                then begin
                    set_brush_color(clLime);
                    display('  sel '+ Self.m_name);
                  end;
              if m_font_height<> 0
                then begin
                    set_font_height(m_font_height);
                    display('  font '+ Self.m_name+ ' '+ IntToStr(m_font_height));
                  end;
              l_width:= f_character_width(Self.m_name[1]);
              l_font_height:= p_c_client_region_drawing.f_get_font_height;

              draw_text(l_x, l_y, Self.m_name[1]);

              restore_graphic_context;
            end
          else
            with p_c_client_region_drawing do
            begin
              l_width:= f_character_width(Self.m_name[1]);
              l_font_height:= f_get_font_height;

              draw_text(l_x, l_y, Self.m_name[1]);
            end;

        display('draw '+ f_display_glyph+ Format(' [%3d %3d h%3d fh%3d  w=%2d fw=%2d]',
            [l_x, l_y, m_font_height, l_font_height,
             m_width, l_width]));
      end; // draw

    procedure c_character_glyph.change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing);
      begin
        if m_is_selected
          then begin
              with p_c_client_region_drawing do
              begin
                // -- change the font height
                m_font_height:= f_get_font_height;
                display('change '+ f_display_glyph);
                display('  fon  '+ IntToStr(f_get_font_height));
                // -- the current canvas has the font height set already
                // set_font_height(m_font_height);

                m_height:= f_character_height(Self.m_name[1]);
                m_width:= f_character_width(Self.m_name[1]);
                display('  =>   '+ f_display_glyph);
              end;
            end
          else display('no_change '+ f_display_glyph);
      end; // change_selected_character_font

    procedure c_character_glyph.accept_visitor(p_c_visitor: c_abstract_visitor);
      begin
        // display('> c_character.accept_visitor');
        p_c_visitor.visit_character(Self);
        // display('< c_character.accept_visitor');
      end; // accept_visitor

    // -- c_rectangle_glyph

    Constructor c_rectangle_glyph.create_rectangle_glyph(p_name: String;
        p_c_parent_glyph: c_glyph; p_width, p_height: Integer);
      begin
        Inherited create_glyph(p_name, p_c_parent_glyph);
        m_width:= p_width+ 4; m_height:= p_height;
      end; // create_rectangle_glyph

    procedure c_rectangle_glyph.draw(p_c_client_region_drawing: c_client_region_drawing);
      var l_x, l_y: Integer;
          l_font_height, l_width: Integer;
      begin
        l_x:= f_c_parent_glyph.m_x+ m_x;
        l_y:= f_c_parent_glyph.m_y+ m_y;

        with p_c_client_region_drawing do
        begin
          save_graphic_context;
          set_pen_color(clAqua);
          // display(f_display_wh_rectangle(l_x, l_y, m_width, m_height));
          draw_rectangle(l_x- 1, l_y- 1, m_width+ 2, m_height+ 2);
          restore_graphic_context;
        end;

        if m_is_selected or (m_font_height<> 0)
          then
            with p_c_client_region_drawing do
            begin
              save_graphic_context;

              if Self.m_is_selected
                then begin
                    set_brush_color(clLime);
                    display('  sel '+ Self.m_name);
                  end;
              if m_font_height<> 0
                then begin
                    set_font_height(m_font_height);
                    display('  font '+ Self.m_name+ ' '+ IntToStr(m_font_height));
                  end;
              l_width:= f_character_width(Self.m_name[1]);
              l_font_height:= p_c_client_region_drawing.f_get_font_height;

              draw_text(l_x, l_y, Self.m_name[1]);

              restore_graphic_context;
            end
          else
            with p_c_client_region_drawing do
            begin
              l_width:= f_character_width(Self.m_name[1]);
              l_font_height:= f_get_font_height;

              draw_text(l_x, l_y, Self.m_name[1]);
            end;

        display('draw '+ f_display_glyph+ Format(' [%3d %3d h%3d fh%3d  w=%2d fw=%2d]',
            [l_x, l_y, m_font_height, l_font_height,
             m_width, l_width]));
      end; // draw

    procedure c_rectangle_glyph.change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing);
      begin
        if m_is_selected
          then begin
              with p_c_client_region_drawing do
              begin
                // -- change the font height
                m_font_height:= f_get_font_height;
                display('change '+ f_display_glyph);
                display('  fon  '+ IntToStr(f_get_font_height));
                // -- the current canvas has the font height set already
                // set_font_height(m_font_height);

                m_height:= f_character_height(Self.m_name[1])+ k_rectangle_width_fat;
                m_width:= f_character_width(Self.m_name[1]);
                display('  =>   '+ f_display_glyph);
              end;
            end
          else display('no_change '+ f_display_glyph);
      end; // change_selected_character_font

    // -- c_bitmap_glyph

    Constructor c_bitmap_glyph.create_bitmap_glyph(p_name: String;
        p_c_parent_glyph: c_glyph; p_width, p_height: Integer;
        p_bitmap_name: String);
      begin
        Inherited create_glyph(p_name, p_c_parent_glyph);

        m_c_bitmap:= tBitmap.Create;
        m_c_bitmap.LoadFromFile(p_bitmap_name);
        m_width:= m_c_bitmap.Width; m_height:= m_c_bitmap.Height;
      end; // create_bitmap_glyph

    procedure c_bitmap_glyph.change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing);
      begin
      end; // change_selected_character_font

    procedure c_bitmap_glyph.draw(p_c_client_region_drawing: c_client_region_drawing);
      begin
        display('draw '+ f_display_glyph);
        with p_c_client_region_drawing do
          draw_bitmap(f_absolute_x, f_absolute_y, m_c_bitmap);
      end; // draw

    Destructor c_bitmap_glyph.Destroy;
      begin
        m_c_bitmap.Free;

        Inherited;
      end; // Destroy

    begin // u_c_character_glyph
    end. // u_c_character_glyph
