// 002 u_c_page_glyph
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- a page is composed of rows

(*$r+*)

unit u_c_page_glyph;
  interface
    uses Classes, Windows
        , ExtCtrls, u_c_basic_object
        , u_c_glyph
        , u_c_row_glyph
        , u_c_row_justification

        ;

    type c_page_glyph= class(c_composite_glyph)
                         Constructor create_page_glyph(p_name: String;
                             p_c_parent_glyph: c_glyph;
                             p_left, p_top, p_width_max, p_height_max: Integer);

                         procedure compute_placement; Override;

                         procedure list_glyphs(p_c_strings: tStrings);

                         function f_c_append_row_glyph: c_row_glyph;
                         function f_contains_glyph_leaves: Boolean;

                         // -- visitors
                         function f_count_glyphs: Integer;
                         procedure check_spelling(p_c_mistake_list: tStringList);

                           procedure _format_row(p_width: Integer;
                               p_c_row_justification_strategy: c_abstract_row_justification_strategy);
                         procedure format_row_to_the_left(p_width: Integer);
                         procedure format_row_to_the_right(p_width: Integer);
                         procedure format_row_both_sides(p_width: Integer);

                         procedure format_page(p_width: Integer);

                         procedure free_row_structures;

                         Destructor Destroy; Override;
                       end; // c_page_glyph

  implementation
    uses SysUtils, Math, u_c_display
        , u_c_glyph_kinds
        , u_c_iterator_kinds
        , u_c_glyph_count_visitor
        , u_c_spelling_checker_visitor

        , u_c_justification_strategy_2
        ;

    // -- c_page_glyph

    Constructor c_page_glyph.create_page_glyph(p_name: String;
        p_c_parent_glyph: c_glyph;
        p_left, p_top, p_width_max, p_height_max: Integer);
      begin
        Inherited create_composite_glyph(p_name, p_c_parent_glyph);
        m_x:= p_left; m_y:= p_top; m_width_max:= p_width_max; m_height_max:= p_height_max;
      end; // create_page_glyph

    procedure c_page_glyph.compute_placement;
      var l_row_glyph_index: Integer;
          l_current_y: Integer;
          l_width_max: Integer;
      begin
        display('> page.compute_placement '+ m_name);

        // l_current_y:= m_y;
        // -- for RELATIVE coordinate, start at 0
        // l_current_y:= k_y_margin;
        l_current_y:= m_y;
        l_width_max:= 0;

        for l_row_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_row_glyph_index) do
          begin
            m_y:= l_current_y;
            display('row_y ='+ IntToStr(m_y));
            m_x:= Self.m_x;

            // -- recursive call to get all character's Sum(width) and Max(Height)
            // -- from the ROW
            compute_placement;

            Inc(l_current_y, m_height);
            l_width_max:= Max(l_width_max, m_width);
          end;

        m_height:= l_current_y;
        m_width:= l_width_max;

        display('< page.compute_placement ');
      end; // compute_placement

    function c_page_glyph.f_c_append_row_glyph: c_row_glyph;
        // -- needs this to append a new line when caret is
        // --  after the last line
      begin
        Result:= c_row_glyph.create_row_glyph('row_'+ IntToStr(f_glyph_count), Self);
        append_glyph(Result);
      end; // f_c_append_row_glyph

    function c_page_glyph.f_contains_glyph_leaves: Boolean;
        // -- avoid some bugs if there are no leaves
        // --   could use an iterator

      procedure check_recursive(p_c_glyph: c_glyph);
        var l_glyph_index: Integer;
        begin
          if p_c_glyph is c_leaf_glyph
            then begin
                Result:= True;
                Exit;
              end;

          if p_c_glyph is c_composite_glyph
            then
              with c_composite_glyph(p_c_glyph) do
                for l_glyph_index:= 0 to f_glyph_count- 1 do
                begin
                  check_recursive(f_c_glyph(l_glyph_index));
                  if Result
                    then Break;
                end;
        end; // check_recursive

      begin // f_contains_glyph_leaves
        Result:= False;
        check_recursive(Self);
      end; // f_contains_glyph_leaves

    procedure c_page_glyph.list_glyphs(p_c_strings: tStrings);
        // -- an iterator exercise
      var l_c_leaf_iterator: c_leaf_iterator;
          l_c_current_glyph: c_glyph;
          l_glyph_count: Integer;
      begin
        display('> list_glyphs');
        l_c_leaf_iterator:= c_leaf_iterator.create_leaf_iterator('leaf_iter', Self);

        with l_c_leaf_iterator do
        begin
          m_trace_iterator:= True;
          m_trace_iterator:= False;

          goto_first_glyph;
          l_glyph_count:= 0;
          while not f_is_done do
          begin
            l_c_current_glyph:= f_c_current_glyph;
            if l_c_current_glyph= Nil
              then Break;
            display(Format('%3d ', [l_glyph_count])+ l_c_current_glyph.m_name);
            Inc(l_glyph_count);
            // p_c_strings.Add(l_c_current_glyph.m_name);
            goto_next_glyph;
          end; // while not f_is_done
          Free;
        end; // with l_c_leaf_iterator
        display('< list_glyphs');
      end; // list_glyphs

    function c_page_glyph.f_count_glyphs: Integer;
        // -- simply count the glyphs which are non blank
      var l_c_leaf_iterator: c_leaf_iterator;
          l_c_current_glyph: c_glyph;
          l_c_glyph_count_visitor: c_glyph_count_visitor;
      begin
        display('> count');

        l_c_leaf_iterator:= c_leaf_iterator.create_leaf_iterator('leaf_iter', Self);
        l_c_glyph_count_visitor:= c_glyph_count_visitor.create_glyph_count_visitor('count');

        with l_c_leaf_iterator do
        begin
          m_trace_iterator:= False;

          goto_first_glyph;
          while not f_is_done do
          begin
            l_c_current_glyph:= f_c_current_glyph;
            // display('cnt '+ l_c_current_glyph.f_display_glyph);

            l_c_current_glyph.accept_visitor(l_c_glyph_count_visitor);

            goto_next_glyph;
          end; // while not f_is_done

          Result:= l_c_glyph_count_visitor.m_glyph_count;

          Free;
        end; // with l_c_leaf_iterator
        display('< count');
      end; // count_glyphs

    procedure c_page_glyph.check_spelling(p_c_mistake_list: tStringList);
      var l_c_leaf_iterator: c_leaf_iterator;
          l_c_current_glyph: c_glyph;
          l_c_spelling_checker_visitor: c_spelling_checker_visitor;
          l_c_current_row, l_c_previous_row: c_row_glyph;
      begin
        display('> spell');
        l_c_leaf_iterator:= c_leaf_iterator.create_leaf_iterator('leaf_iter', Self);
        l_c_spelling_checker_visitor:=
            c_spelling_checker_visitor.create_spelling_checker_visitor('spell',
            p_c_mistake_list);

        // l_c_current_row:= Nil;
        l_c_previous_row:= Nil;

        with l_c_leaf_iterator do
        begin
          m_trace_iterator:= False;

          goto_first_glyph;
          while not f_is_done do
          begin
            l_c_current_glyph:= f_c_current_glyph;

            if l_c_current_glyph is c_character_glyph
              then begin
                  l_c_current_row:= l_c_current_glyph.f_c_parent_glyph as c_row_glyph;
                  if (l_c_previous_row<> Nil) and (l_c_current_row<> l_c_previous_row)
                    then l_c_previous_row.accept_visitor(l_c_spelling_checker_visitor);
                  l_c_previous_row:= l_c_current_row;
                end;

            l_c_current_glyph.accept_visitor(l_c_spelling_checker_visitor);

            goto_next_glyph;
          end; // while not f_is_done

          l_c_previous_row.accept_visitor(l_c_spelling_checker_visitor);

          Free;
        end; // with l_c_leaf_iterator
        display('< spell');
      end; // check_spelling

    // -- strategy

    procedure c_page_glyph._format_row(p_width: Integer;
        p_c_row_justification_strategy: c_abstract_row_justification_strategy);
      var l_c_row_iterator: c_composite_iterator;
          l_c_row_glyph: c_row_glyph;
      begin
        l_c_row_iterator:= c_composite_iterator.create_composite_iterator('row_iter', Self);

        while not l_c_row_iterator.f_is_done do
        begin
          l_c_row_glyph:= c_row_glyph(l_c_row_iterator.f_c_current_glyph);

          l_c_row_glyph.set_row_justification_strategy(p_c_row_justification_strategy);
          p_c_row_justification_strategy.set_row(l_c_row_glyph);

          with l_c_row_glyph do
          begin
            display_line;
            display(f_display_glyph);
            p_c_row_justification_strategy.justify_row(p_width);
          end; // with l_c_row_glyph

          l_c_row_iterator.goto_next_glyph;
        end; // while not l_c_row_iterator
      end; // _format_row

    procedure c_page_glyph.format_row_to_the_left(p_width: Integer);
      var l_c_row_justification_strategy: c_abstract_row_justification_strategy;
      begin
        display('> format_row_to_the_left');
        l_c_row_justification_strategy:= c_left_row_justification_strategy.create_left_row_justification_strategy('row_right');
        _format_row(p_width, l_c_row_justification_strategy);
        l_c_row_justification_strategy.Free;
        display('< format_row_to_the_left');
      end; // format_row_to_the_left

    procedure c_page_glyph.format_row_to_the_right(p_width: Integer);
      var l_c_row_justification_strategy: c_abstract_row_justification_strategy;
      begin
        display('> format_row_to_the_right');
        l_c_row_justification_strategy:= c_right_row_justification_strategy.create_right_row_justification_strategy('row_right');
        _format_row(p_width, l_c_row_justification_strategy);
        l_c_row_justification_strategy.Free;
        display('< format_row_to_the_right');
      end; // format_row_to_the_right

    procedure c_page_glyph.format_row_both_sides(p_width: Integer);
      var l_c_both_justification_strategy: c_both_row_justification_strategy;
      begin
        display('> format_row_both_sides');
        l_c_both_justification_strategy:= c_both_row_justification_strategy.create_both_row_justification_strategy('both');
        _format_row(p_width, l_c_both_justification_strategy);
        l_c_both_justification_strategy.Free;
        display('< format_row_both_sides');
      end; // format_row_both_sides

    procedure c_page_glyph.format_page(p_width: Integer);
      var l_c_page_left_justification_strategy: c_abstract_justification_strategy_2;
      begin
        display('> format_page');

        l_c_page_left_justification_strategy:=
            c_left_justification_strategy_2.create_left_justification_strategy_2('both', Self);
        l_c_page_left_justification_strategy.do_justify(p_width);
        l_c_page_left_justification_strategy.Free;

        display('< format_page');
      end; // format_page

    procedure c_page_glyph.free_row_structures;
        // -- necessary for the page cloning during justification strategies
      var l_row_index: Integer;
      begin
        for l_row_index:= 0 to f_glyph_count- 1 do
        begin
          (f_c_glyph(l_row_index) as c_composite_glyph).nil_structure;
          (f_c_glyph(l_row_index) as c_composite_glyph).free_structure;
        end;
        nil_structure;
        free_structure;
      end; // free_row_structures

    Destructor c_page_glyph.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin
    end.
