// 002 u_c_row_glyph
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_row_glyph;
  interface
    uses Classes, Windows, ExtCtrls, u_c_basic_object
       , u_c_glyph
       , u_c_glyph_kinds
       , u_c_visitor
       , u_c_row_justification
       , u_c_justification_strategy_2
       ;

    type c_row_glyph= class(c_composite_glyph)
                        m_c_row_justification_strategy_ref: c_abstract_row_justification_strategy;
                        m_c_justification_strategy_ref: c_abstract_justification_strategy_2;

                        Constructor create_row_glyph(p_name: String;
                            p_c_parent_glyph: c_glyph);

                        function f_absolute_y: Integer; Override;

                        procedure compute_placement; Override;

                        procedure accept_visitor(p_c_visitor: c_abstract_visitor); Override;

                        procedure set_row_justification_strategy(p_c_row_justification_strategy_ref: c_abstract_row_justification_strategy);
                        procedure set_justification_strategy(p_c_justification_strategy_ref: c_abstract_justification_strategy_2);

                        procedure insert_leaf_glyph(p_c_leaf_glyph: c_leaf_glyph;
                            p_insert_index: Integer);

                        Destructor Destroy; Override;
                      end; // c_row_glyph

  implementation
    uses SysUtils, Math, u_c_display;

    // -- c_row_glyph

    Constructor c_row_glyph.create_row_glyph(p_name: String;
        p_c_parent_glyph: c_glyph);
      begin
        Inherited create_composite_glyph(p_name, p_c_parent_glyph);
      end; // create_row_glyph

    function c_row_glyph.f_absolute_y: Integer;
      begin
        Result:= m_y
      end; // f_absolute_y

    procedure c_row_glyph.compute_placement;
      var l_character_glyph_index: Integer;
          l_current_x: Integer;
          l_height_max: Integer;
      begin
        display('> row.compute_placement '+ m_name);

        // l_current_x:= m_x;
        // -- for relative coordinates, use 0 as a start
        l_current_x:= k_x_margin;
        l_height_max:= 0;

        for l_character_glyph_index:= 0 to f_glyph_count- 1 do
          with f_c_glyph(l_character_glyph_index) do
          begin
            m_x:= l_current_x;
            // m_y:= 0;
            // display('  y ='+ IntToStr(m_y ));

            // -- compute each row element width and height
            compute_placement;

            Inc(l_current_x, m_width+ k_caret_width);
            l_height_max:= Max(l_height_max, m_y+ m_height);
          end;

        m_width:= l_current_x;
        m_height:= l_height_max;
        display('< row.compute_placement ');
      end; // compute_placement

    procedure c_row_glyph.accept_visitor(p_c_visitor: c_abstract_visitor);
      begin
        // display('row.accept_visitor');
        p_c_visitor.visit_row(Self);
      end; // accept_visitor

    procedure c_row_glyph.set_row_justification_strategy(p_c_row_justification_strategy_ref: c_abstract_row_justification_strategy);
      begin
        m_c_row_justification_strategy_ref:= p_c_row_justification_strategy_ref;
      end; // set_row_justification_strategy

    procedure c_row_glyph.set_justification_strategy(p_c_justification_strategy_ref: c_abstract_justification_strategy_2);
      begin
        m_c_justification_strategy_ref:= p_c_justification_strategy_ref;
      end; // set_justification_strategy

(*
    function c_row_glyph.f_c_insert_leaf_glyph(p_character: Char; p_insert_mode
        p_insert_index, p_width, p_height: Integer): c_leaf_glyph;
      begin
        Result:= c_character_glyph.create_character_glyph(p_character,
          Self, p_width, p_height);

        // -- append or insert
        if p_insert_index>= f_glyph_count
          then append_glyph(Result)
          else insert_glyph(p_insert_index, Result)
      end; // f_c_insert_leaf_glyph
*)
    procedure c_row_glyph.insert_leaf_glyph(p_c_leaf_glyph: c_leaf_glyph;
        p_insert_index: Integer);
      begin
        // -- append or insert
        if p_insert_index>= f_glyph_count
          then append_glyph(p_c_leaf_glyph)
          else insert_glyph(p_insert_index, p_c_leaf_glyph)
      end; // insert_leaf_glyph

    Destructor c_row_glyph.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin
    end.
