// 002 u_c_glyph_border_decorator_2
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_glyph_border_decorator_2;
  interface
    uses Windows, Classes, Graphics, ExtCtrls
        , u_c_bridge_drawings
        , u_c_glyph
        , u_c_page_glyph
        ;

    type c_abstract_border_decorator= class(c_page_glyph)
                                        // -- the content of the decorator: either a page
                                        // --   or a previously added decorator
                                        _m_c_enclosed_glyph: c_page_glyph;

                                        Constructor create_abstract_border_decorator(p_name: String;
                                            p_c_parent_glyph: c_glyph;
                                            p_c_enclosed_glyph: c_page_glyph);

                                        function f_display_glyph: String; Override;
                                        procedure display_glyph; Override;

                                        procedure draw(p_c_client_region_drawing: c_client_region_drawing); Override;

                                        // -- replace ALL the functions called for m_c_root_page
                                        // --   the c_page_glyph proper AND those of its c_composite_glyph or c_glyph_ancestors

                                        procedure append_glyph(p_c_glyph: c_glyph); Override;
                                        procedure select_in_rectangle(p_absolute_rectangle: tRect); Override;
                                        procedure compute_placement; Override;
                                        procedure change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing); Override;
                                        // procedure shrink(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer); Override;
                                        procedure list_glyphs(p_c_strings: tStrings);
                                      end; // c_abstract_border_decorator

  implementation
    uses SysUtils, u_c_display;

    // -- c_abstract_border_decorator

    Constructor c_abstract_border_decorator.create_abstract_border_decorator(p_name: String;
        p_c_parent_glyph: c_glyph;
        p_c_enclosed_glyph: c_page_glyph);
      begin
        // -- initialize with the enclosing rectangle of the previous enclosed
        with p_c_enclosed_glyph do
          Inherited create_page_glyph(p_name, p_c_parent_glyph,
              f_absolute_x, f_absolute_y, m_width_max, m_height_max);
        // Inherited create_composite_glyph(p_name, Nil);
        _m_c_enclosed_glyph:= p_c_enclosed_glyph;
      end; // create_abstract_border_decorator

    // -- the only "TRUE" decorator method

    procedure c_abstract_border_decorator.draw(p_c_client_region_drawing: c_client_region_drawing);
      begin
        if _m_c_enclosed_glyph= Nil
          then display_bug_halt('no_enclosed '+ m_name);
        // -- forward to the enclosed "glyph"
        _m_c_enclosed_glyph.draw(p_c_client_region_drawing);
      end; // draw

    // -- delegation of the enclosed methods

    function c_abstract_border_decorator.f_display_glyph: String;
      begin
        Result:= m_name;
      end; // f_display_glyph

    procedure c_abstract_border_decorator.display_glyph;
      begin
        _m_c_enclosed_glyph.display_glyph;
      end; // display_glyph

    procedure c_abstract_border_decorator.append_glyph(p_c_glyph: c_glyph);
      begin
        _m_c_enclosed_glyph.append_glyph(p_c_glyph);
      end; // append_glyph

    procedure c_abstract_border_decorator.select_in_rectangle(p_absolute_rectangle: tRect);
      begin
        _m_c_enclosed_glyph.select_in_rectangle(p_absolute_rectangle);
      end; // select_in_rectangle

    procedure c_abstract_border_decorator.compute_placement;
      begin
        _m_c_enclosed_glyph.compute_placement;
      end; // compute_placement

    procedure c_abstract_border_decorator.change_selected_character_font(p_c_client_region_drawing: c_client_region_drawing);
      begin
        _m_c_enclosed_glyph.change_selected_character_font(p_c_client_region_drawing);
      end; // change_selected_character_font

    procedure c_abstract_border_decorator.list_glyphs(p_c_strings: tStrings);
      begin
        _m_c_enclosed_glyph.list_glyphs(p_c_strings);
      end; // list_glyphs

    begin // u_c_abstract_border_decorator
    end. // u_c_abstract_border_decorator



                                        // -- all the c_page_glyph methods
                                        procedure compute_placement;
                                        procedure list_glyphs(p_c_strings: tStrings);
                                        function f_c_append_row_glyph: c_row_glyph;
                                        function f_contains_glyph_leaves: Boolean;
                                        function f_count_glyphs: Integer;
                                        procedure check_spelling(p_c_mistake_list: tStringList);
                                        procedure _format_row(p_width: Integer;
                                            p_c_row_justification_strategy: c_abstract_row_justification_strategy);
                                        procedure format_row_to_the_left(p_width: Integer);
                                        procedure format_row_to_the_right(p_width: Integer);
                                        procedure format_row_both_sides(p_width: Integer);
                                        procedure format_page(p_width: Integer);
                                        procedure free_row_structures;

