// 002 u_c_iterator_kinds
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_iterator_kinds;
  interface
    uses Classes, u_c_basic_object
        , u_c_glyph
        , u_c_iterator
        , u_c_integer_stack
        ;

    type c_composite_iterator= class(c_abstract_iterator)
                                 _m_c_composite_glyph_ref: c_composite_glyph;
                                 m_glyph_index: Integer;

                                 Constructor create_composite_iterator(p_name: String;
                                     p_c_glyph_ref: c_glyph);
                                 function f_display_iterator: String; Override;
                                 procedure goto_first_glyph; Override;
                                 procedure goto_next_glyph; Override;
                                 function f_is_done: Boolean; Override;
                                 function f_c_current_glyph: c_glyph; Override;
                               end; // c_composite_iterator

         c_composite_iterator_stack= // "integer_stack" list
                                     Class(c_basic_object)
                                       _m_c_composite_iterator_stack: tList;

                                       Constructor create_composite_iterator_stack(p_name: String);

                                       function f_c_composite_iterator_stack(p_stack_index: Integer): c_composite_iterator;
                                       procedure push_composite_iterator(p_c_composite_iterator: c_composite_iterator);
                                       function f_c_top_of_stack_composite_iterator: c_composite_iterator;
                                       function f_c_pop_composite_iterator: c_composite_iterator;
                                       procedure clear_stack;
                                       function f_is_empty: Boolean;
                                       procedure display_composite_iterator_stack;

                                       Destructor Destroy; Override;
                                     end; // c_composite_iterator_stack

         t_iterator_type= (e_leaf_iterator, e_word_iterator);

         c_leaf_iterator= class(c_abstract_iterator)
                               m_c_composite_iterator_stack: c_composite_iterator_stack;
                               m_trace_iterator: Boolean;

                               Constructor create_leaf_iterator(p_name: String;
                                   p_c_glyph_ref: c_glyph);
                               procedure trace_iterator(p_text: String);
                               procedure goto_first_glyph; Override;
                               procedure goto_next_glyph; Override;
                               function f_is_done: Boolean; Override;
                               function f_c_current_glyph: c_glyph; Override;
                               Destructor Destroy; Override;
                             end; // c_leaf_iterator

         c_null_iterator= class(c_abstract_iterator)
                            Constructor create_null_iterator(p_name: String;
                                p_c_glyph_ref: c_glyph);
                            function f_is_done: Boolean; Override;
                          end; // c_null_iterator

  implementation
    uses SysUtils, u_c_display
        , u_strings
        ;

    // -- c_composite_iterator

    Constructor c_composite_iterator.create_composite_iterator(p_name: String;
        p_c_glyph_ref: c_glyph);
      begin
        Inherited create_abstract_iterator(p_name, p_c_glyph_ref);

        // -- avoid continuous casts
        _m_c_composite_glyph_ref:= c_composite_glyph(_m_c_glyph_ref);
      end; // create_composite_iterator

    function c_composite_iterator.f_display_iterator: String;
      begin
        Result:= m_name;
      end; // f_display_iterator

    procedure c_composite_iterator.goto_first_glyph;
      begin
        m_glyph_index:= 0;
      end; // f_c_first_glyph

    procedure c_composite_iterator.goto_next_glyph;
      begin
        Inc(m_glyph_index);
      end; // f_c_next_glyph

    function c_composite_iterator.f_is_done: Boolean;
      begin
        Result:= m_glyph_index>= _m_c_composite_glyph_ref.f_glyph_count;
        // display('is_done '+ IntToStr(m_glyph_index)
        //  + ' '+ IntToStr(_m_c_composite_glyph_ref.f_glyph_count)
        //  + ' '+ f_display_TF(Result));
      end; // f_is_done

    function c_composite_iterator.f_c_current_glyph: c_glyph;
      begin
        Result:= _m_c_composite_glyph_ref.f_c_glyph(m_glyph_index);
      end; // f_c_current_glyph

    // -- c_composite_iterator_stack

    Constructor c_composite_iterator_stack.create_composite_iterator_stack(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        _m_c_composite_iterator_stack:= tList.Create;
      end; // create_composite_iterator_stack_line

    function c_composite_iterator_stack.f_c_composite_iterator_stack(p_stack_index: Integer): c_composite_iterator;
      begin
        Result:= c_composite_iterator(_m_c_composite_iterator_stack[p_stack_index]);
      end; //  f_composite_iterator_stack

    procedure c_composite_iterator_stack.push_composite_iterator(p_c_composite_iterator: c_composite_iterator);
      begin
        _m_c_composite_iterator_stack.Add(tObject(p_c_composite_iterator));
      end; // push_composite_iterator

    function c_composite_iterator_stack.f_c_top_of_stack_composite_iterator: c_composite_iterator;
      begin
        Result:= f_c_composite_iterator_stack(_m_c_composite_iterator_stack.Count- 1);
      end; // f_top_composite_iterator

    function c_composite_iterator_stack.f_c_pop_composite_iterator: c_composite_iterator;
      begin
        Result:= f_c_top_of_stack_composite_iterator;
        with _m_c_composite_iterator_stack do
          Delete(Count- 1);
      end; // f_pop_of_stack_composite_iterator

    procedure c_composite_iterator_stack.display_composite_iterator_stack;
      var l_composite_iterator_stack: Integer;
      begin
        display('stack '+ m_name+ ' ('+ IntToStr(_m_c_composite_iterator_stack.Count)+ ')');
        for l_composite_iterator_stack:= 0 to _m_c_composite_iterator_stack.Count- 1 do
          display(f_c_composite_iterator_stack(l_composite_iterator_stack).f_display_iterator);
      end; // display_composite_iterator_stack

    function c_composite_iterator_stack.f_is_empty: Boolean;
      begin
        Result:= _m_c_composite_iterator_stack.Count= 0;
      end; // f_is_empty

    procedure c_composite_iterator_stack.clear_stack;
      var l_composite_iterator_stack: Integer;
      begin
        for l_composite_iterator_stack:= 0 to _m_c_composite_iterator_stack.Count- 1 do
          f_c_composite_iterator_stack(l_composite_iterator_stack).Free;
      end; // clear_stack

    Destructor c_composite_iterator_stack.Destroy;
      begin
        clear_stack;
        _m_c_composite_iterator_stack.Free;
        Inherited;
      end; // Destroy

    // -- c_leaf_iterator

    Constructor c_leaf_iterator.create_leaf_iterator(p_name: String;
        p_c_glyph_ref: c_glyph);
      begin
        Inherited create_abstract_iterator(p_name, p_c_glyph_ref);

        m_c_composite_iterator_stack:= c_composite_iterator_stack.create_composite_iterator_stack('iter_stack');
      end; // create_leaf_iterator

    procedure c_leaf_iterator.trace_iterator(p_text: String);
      var l_stack: String;
      begin
        if m_trace_iterator
          then begin
              if p_text[1] in ['<', '=']
                then begin
                    l_stack:= ' '+ m_name+ ' cnt ='+ IntToStr(m_c_composite_iterator_stack._m_c_composite_iterator_stack.Count);
                    if m_c_composite_iterator_stack._m_c_composite_iterator_stack.Count> 0
                      then l_stack:= l_stack
                          + ' [ix ='
                          + IntToStr(m_c_composite_iterator_stack.f_c_top_of_stack_composite_iterator.m_glyph_index)
                          + ']';
                  end
                else l_stack:= '';

              display(p_text+ l_stack);
            end;
      end; // trace_iterator

    procedure c_leaf_iterator.goto_first_glyph;
      var l_c_composite_iterator: c_composite_iterator;
          l_c_current_glyph: c_glyph;
          l_c_child_iterator: c_composite_iterator;
      begin
        trace_iterator('> c_elem.goto_first_glyph');
        l_c_composite_iterator:= c_composite_iterator.create_composite_iterator('compo', _m_c_glyph_ref);
        l_c_composite_iterator.goto_first_glyph;

        m_c_composite_iterator_stack.clear_stack;
        m_c_composite_iterator_stack.push_composite_iterator(l_c_composite_iterator);

        // -- even for the first, must drill down to get the first leaf
        with m_c_composite_iterator_stack do
        begin
          l_c_current_glyph:= f_c_top_of_stack_composite_iterator.f_c_current_glyph;
          l_c_child_iterator:= c_composite_iterator.create_composite_iterator('child_iter', l_c_current_glyph);
          l_c_child_iterator.goto_first_glyph;
          push_composite_iterator(l_c_child_iterator);
        end;  

        trace_iterator('< c_elem.goto_first_glyph');
      end; // goto_first_glyph

    procedure c_leaf_iterator.goto_next_glyph;
      var l_c_current_glyph: c_glyph;
          l_c_child_iterator: c_composite_iterator;
          l_c_composite_iterator: c_composite_iterator;
      begin
        trace_iterator('> c_elem.goto_next_glyph ');

        with m_c_composite_iterator_stack do
        begin
          if f_is_empty
            then display_bug_halt('empty');

          l_c_current_glyph:= f_c_top_of_stack_composite_iterator.f_c_current_glyph;

          if l_c_current_glyph= Nil
            then display_bug_halt('nil');

          if l_c_current_glyph is c_composite_glyph
            then begin
                trace_iterator('compo');
                l_c_child_iterator:= c_composite_iterator.create_composite_iterator('child_iter', l_c_current_glyph);
                l_c_child_iterator.goto_first_glyph;
                push_composite_iterator(l_c_child_iterator);
                // display_composite_iterator_stack;
              end
            else begin
                trace_iterator('simple');
                // -- this is a simple glyph
                Repeat
                  l_c_composite_iterator:= f_c_top_of_stack_composite_iterator;
                  // -- get the next (to toggle done eventually)
                  l_c_composite_iterator.goto_next_glyph;

                  if l_c_composite_iterator.f_is_done
                    then begin
                        // -- pop and free
                        trace_iterator('pop');
                        f_c_pop_composite_iterator.Free;
                        l_c_composite_iterator:= Nil;

                        if not f_is_empty
                          then begin
                              trace_iterator('stack_not_empty');
                              l_c_composite_iterator:= f_c_top_of_stack_composite_iterator;
                              if not l_c_composite_iterator.f_is_done
                                then begin
                                    trace_iterator('iter_not_done');
                                    l_c_composite_iterator.goto_next_glyph;

                                    if not l_c_composite_iterator.f_is_done
                                      then // -- recurse
                                           Self.goto_next_glyph
                                      else l_c_composite_iterator:= Nil;
                                  end;
                            end;
                      end;
                until f_is_empty or (l_c_composite_iterator<> Nil);
              end;
        end; // with

        trace_iterator('< c_elem.goto_next_glyph ');
      end; // goto_next_glyph

    function c_leaf_iterator.f_is_done: Boolean;
      begin
        Result:= m_c_composite_iterator_stack.f_is_empty;
        trace_iterator('= c_elem.f_is_done '+ f_display_TF(Result));
      end; // f_is_done

    function c_leaf_iterator.f_c_current_glyph: c_glyph;
      begin
        Result:= m_c_composite_iterator_stack.f_c_top_of_stack_composite_iterator.f_c_current_glyph;
      end; // f_c_current_glyph

    Destructor c_leaf_iterator.Destroy;
      begin
        m_c_composite_iterator_stack.Free;
        Inherited;
      end; // Destroy

    // -- c_null_iterator

    Constructor c_null_iterator.create_null_iterator(p_name: String;
        p_c_glyph_ref: c_glyph);
      begin
        Inherited create_abstract_iterator(p_name, p_c_glyph_ref);
      end; // create_null_iterator

    function c_null_iterator.f_is_done: Boolean;
        // -- ALWAYS return TRUE
      begin
        Result:= True;
      end; // f_is_done

    begin // u_c_leaf_iterator
    end. // u_c_preorder_iterator





