// 002 u_c_justification_strategy_2
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- inserts columns and rows when overflows

(*$r+*)

unit u_c_justification_strategy_2;
  interface
    uses u_c_basic_object;

    type c_abstract_justification_strategy_2= class(c_basic_object)
                                                // -- m_c_page_glyph_ref: c_page_glyph;
                                                // --  but circular reference
                                                m_c_page_glyph_ref: tObject;

                                                constructor create_abstract_justification_strategy_2(p_name: String;
                                                    p_c_page_glyph_ref: tObject);
                                                procedure do_justify(p_width: Integer); Virtual; Abstract;
                                              end; // c_abstract_justification_strategy_2

         // -- transfer to the next line if spills over
         c_left_justification_strategy_2= class(c_abstract_justification_strategy_2)
                                            Constructor create_left_justification_strategy_2(p_name: String;
                                                p_c_glyph_ref: tObject);
                                            procedure do_justify(p_width: Integer); Override;
                                          end; // c_left_justification_strategy_2

         // -- here the right justification etc.

  implementation
    uses SysUtils, Classes, u_c_display
        , u_c_glyph
        , u_c_iterator_kinds // for justification
        , u_c_page_glyph
        , u_c_row_glyph
        , u_c_glyph_kinds
        ;

    // -- c_abstract_justification_strategy_2

    constructor c_abstract_justification_strategy_2.create_abstract_justification_strategy_2(p_name: String;
        p_c_page_glyph_ref: tObject);
      begin
        Inherited create_basic_object(p_name);
        m_c_page_glyph_ref:= p_c_page_glyph_ref;
      end; // create_abstract_justification_strategy_2

    // -- c_left_justification_strategy_2

    Constructor c_left_justification_strategy_2.create_left_justification_strategy_2(p_name: String;
        p_c_glyph_ref: tObject);
      begin
        Inherited create_abstract_justification_strategy_2(p_name, p_c_glyph_ref);
      end; // create_left_justification_strategy_2

    procedure c_left_justification_strategy_2.do_justify(p_width: Integer);
        // -- called by the c_page_glyph
      var // -- casting
          l_c_current_page_glyph: c_page_glyph;

          l_c_new_page_glyph: c_page_glyph;
          l_c_row_glyph: c_row_glyph;
          l_row_count: Integer;

          l_c_leaf_iterator: c_leaf_iterator;
          l_c_current_glyph: c_glyph;

          l_line_width: Integer;
      begin
        display('> do_justify '+ IntToStr(p_width));
        l_c_current_page_glyph:= m_c_page_glyph_ref as c_page_glyph;
        // -- the page to re build
        with l_c_current_page_glyph do
          l_c_new_page_glyph:= c_page_glyph.create_page_glyph('new_page', Nil,
          m_x, m_y, m_width_max, m_height_max);

        l_c_leaf_iterator:= c_leaf_iterator.create_leaf_iterator('leaf_iter', l_c_current_page_glyph);

        l_row_count:= 0;
        l_c_row_glyph:= c_row_glyph.create_row_glyph('row_'+ IntToStr(l_row_count), l_c_new_page_glyph);
        l_c_new_page_glyph.append_glyph(l_c_row_glyph);
        Inc(l_row_count);

        display('empty_page');
        l_c_new_page_glyph.display_glyph;

        display('create rows');
        with l_c_leaf_iterator do
        begin
          goto_first_glyph;

          l_line_width:= 0;
          while not f_is_done do
          begin
            l_c_current_glyph:= f_c_current_glyph;
            if l_c_current_glyph is c_leaf_glyph
              then begin
                  display('g ='+ l_c_current_glyph.m_name+ ' ['+ IntToStr(l_line_width)+ ']');

                  Inc(l_line_width, l_c_current_glyph.m_width);

                  if l_line_width> p_width
                    then begin
                        display('line. Before_line');
                        l_c_new_page_glyph.display_glyph;

                        // -- create the new line
                        l_c_row_glyph:= c_row_glyph.create_row_glyph('row_'+ IntToStr(l_row_count), l_c_new_page_glyph);
                        l_c_new_page_glyph.append_glyph(l_c_row_glyph);
                        Inc(l_row_count);
                        display('   after_line');
                        l_c_new_page_glyph.display_glyph;

                        l_line_width:= 0;
                      end;

                  // -- whether in old or new line, add the glyph to the line
                  l_c_row_glyph.append_glyph(l_c_current_glyph);
                  // -- AND change the parent (for height placement !
                  l_c_current_glyph._m_c_parent_glyph:= l_c_row_glyph;
                end;

            goto_next_glyph;
          end; // while not f_is_done

          // -- the cleanest would have been to clone each character, free the old
          l_c_new_page_glyph.display_glyph;
          // -- now exchange the structures and nil the old. Hem...
          l_c_current_page_glyph.free_row_structures;

          display('place');
          l_c_current_page_glyph._m_c_glyph_list:= l_c_new_page_glyph._m_c_glyph_list;
          l_c_current_page_glyph.compute_placement;

          // -- now cancel the new page
          // l_c_new_page_glyph.nil_structure;
          // l_c_new_page_glyph.Free;
          // l_c_new_page_glyph.free_row_structures;
          // -- create a bogus list, so that f_count still works
          l_c_new_page_glyph._m_c_glyph_list:= tList.Create;
          l_c_new_page_glyph.Free;

          Free;
        end; // with l_leaf_iterator
        display('< do_justify');
      end; // do_justify

    begin // u_c_abstract_justification_strategy
    end. // u_c_abstract_justification_strategy
