// 002 u_c_row_justification
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- inserts columns and rows when overflows

(*$r+*)

unit u_c_row_justification;
  interface
    uses u_c_basic_object;

    type c_abstract_row_justification_strategy= class(c_basic_object)
                                                   // -- m_c_page_glyph_ref: c_page_glyph;
                                                   // --  but circular reference
                                                   m_c_row_glyph_ref: tObject;

                                                   procedure set_row(p_c_row_glyph_ref: tObject);
                                                   procedure justify_row(p_width: Integer); Virtual; Abstract;
                                                 end; // c_abstract_row_justification_strategy

         // -- push everything to the left
         c_left_row_justification_strategy= class(c_abstract_row_justification_strategy)
                                              Constructor create_left_row_justification_strategy(p_name: String);
                                              procedure justify_row(p_width: Integer); Override;
                                            end; // c_left_row_justification_strategy

         // -- push everything to the right
         c_right_row_justification_strategy= class(c_abstract_row_justification_strategy)
                                               Constructor create_right_row_justification_strategy(p_name: String);
                                               procedure justify_row(p_width: Integer); Override;
                                             end; // c_right_row_justification_strategy

         // -- align to the left AND to the right
         c_both_row_justification_strategy= class(c_abstract_row_justification_strategy)
                                              Constructor create_both_row_justification_strategy(p_name: String);
                                              procedure justify_row(p_width: Integer); Override;
                                            end; // = class(c_abstract_row_justification_strategy)
  implementation
    uses SysUtils, u_c_display
        , u_c_glyph
        , u_c_iterator_kinds // for justification
        ;
        
    // -- c_abstract_row_justification_strategy

    procedure c_abstract_row_justification_strategy.set_row(p_c_row_glyph_ref: tObject);
      begin
        m_c_row_glyph_ref:= p_c_row_glyph_ref;
      end; // set_row

    // -- c_left_row_justification_strategy

    Constructor c_left_row_justification_strategy.create_left_row_justification_strategy(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_left_row_justification_strategy

    procedure c_left_row_justification_strategy.justify_row(p_width: Integer);
        // -- push everything to the left
        // -- p_width is not used
      var l_c_character_iterator: c_composite_iterator;
          l_c_glyph: c_glyph;
          l_running_x: Integer;
      begin
        l_c_character_iterator:= c_composite_iterator.create_composite_iterator('row_iter',
            c_glyph(m_c_row_glyph_ref));

        l_running_x:= 2;
        while not l_c_character_iterator.f_is_done do
        begin
          l_c_glyph:= l_c_character_iterator.f_c_current_glyph;

          with l_c_glyph do
          begin
            m_x:= l_running_x;
            display('left '+ f_display_glyph);
            Inc(l_running_x, m_width);
          end; // with l_c_row_glyph

          l_c_character_iterator.goto_next_glyph;
        end; // while not l_c_character_iterator.f_is_done
      end; // justify_row

    // -- c_right_row_justification_strategy

    Constructor c_right_row_justification_strategy.create_right_row_justification_strategy(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_right_row_justification_strategy

    procedure c_right_row_justification_strategy.justify_row(p_width: Integer);
        // -- push everything to the right

      function f_compute_row_minimum_width: Integer;
        var l_c_character_iterator: c_composite_iterator;
            l_c_glyph: c_glyph;
        begin
          l_c_character_iterator:= c_composite_iterator.create_composite_iterator('row_iter',
              c_glyph(m_c_row_glyph_ref));

          // -- first compute the pixels of the glyphs
          Result:= 0;
          while not l_c_character_iterator.f_is_done do
          begin
            l_c_glyph:= l_c_character_iterator.f_c_current_glyph;
            with l_c_glyph do
            begin
              Inc(Result, m_width);
            end; // with l_c_row_glyph

            l_c_character_iterator.goto_next_glyph;
          end; // while not l_c_character_iterator.f_is_done
          display(Format('fat= %3d', [Result]));
        end; // f_compute_row_minimum_width

      procedure push_to_the_right(p_row_minimum_width: Integer);
        var l_c_character_iterator: c_composite_iterator;
            l_c_glyph: c_glyph;
            l_running_x: Integer;
        begin
          if p_row_minimum_width> p_width
            then begin
                display('*** not_wide_enough');
                Exit;
              end;

          l_c_character_iterator:= c_composite_iterator.create_composite_iterator('row_iter',
              c_glyph(m_c_row_glyph_ref));

          l_running_x:= p_width- p_row_minimum_width;
          while not l_c_character_iterator.f_is_done do
          begin
            l_c_glyph:= l_c_character_iterator.f_c_current_glyph;
            with l_c_glyph do
            begin
              display('  bef '+ f_display_glyph);
              m_x:= l_running_x;
              display('  aft '+ f_display_glyph);
              Inc(l_running_x, m_width);
            end; // with l_c_row_glyph

            l_c_character_iterator.goto_next_glyph;
          end; // while not l_c_character_iterator.f_is_done
        end; // push_to_the_right

      var l_row_minimum_width: Integer;

      begin // format_page
        l_row_minimum_width:= f_compute_row_minimum_width;
        push_to_the_right(l_row_minimum_width);
      end; // format_page

    // -- c_both_row_justification_strategy

    Constructor c_both_row_justification_strategy.create_both_row_justification_strategy(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_both_row_justification_strategy

    procedure c_both_row_justification_strategy.justify_row(p_width: Integer);
         // -- align to the left AND to the right
      var l_glyph_count: Integer;

      function f_compute_total_fat: Integer;
        var l_c_character_iterator: c_composite_iterator;
            l_glyphs_width: Integer;
            l_c_glyph: c_glyph;
        begin
          l_c_character_iterator:= c_composite_iterator.create_composite_iterator('row_iter',
              c_glyph(m_c_row_glyph_ref));

          // -- first compute the pixels of the glyphs
          l_glyphs_width:= 0; l_glyph_count:= 0;
          while not l_c_character_iterator.f_is_done do
          begin
            l_c_glyph:= l_c_character_iterator.f_c_current_glyph;
            Inc(l_glyph_count);
            with l_c_glyph do
            begin
              Inc(l_glyphs_width, m_width);
              // display('current '+ f_display_glyph+ ' ('+ IntToStr(l_glyphs_width)+ ')');
            end; // with l_c_row_glyph

            l_c_character_iterator.goto_next_glyph;
          end; // while not l_c_character_iterator.f_is_done

          // -- comput the per-glyph fat
          // -- the total amount to distribute
          Result:= p_width- l_glyphs_width;
          display(Format('WIDTH %3d   cnt =%3d   fat= %3d', [l_glyphs_width, l_glyph_count, Result]));
        end; // f_compute_total_fat

      procedure recompute_glyph_x(p_fat: Integer);
        var l_c_character_iterator: c_composite_iterator;
            l_c_glyph: c_glyph;
            l_remaining_fat: Integer;
            l_running_x: Integer;
            l_glyph_fat: Integer;
        begin
          l_c_character_iterator:= c_composite_iterator.create_composite_iterator('row_iter',
              c_glyph(m_c_row_glyph_ref));

          l_remaining_fat:= p_fat;
          l_c_character_iterator.goto_first_glyph;

          l_c_glyph:= l_c_character_iterator.f_c_current_glyph;
          // -- the first m_x remains the same
          l_running_x:= l_c_glyph.m_x+ l_c_glyph.m_width;

          // -- start after the first
          l_c_character_iterator.goto_next_glyph;
          Dec(l_glyph_count);

          // -- for the n-1 remaing
          while l_glyph_count> 0 do
          begin
            l_glyph_fat:= l_remaining_fat div l_glyph_count;
            l_c_glyph:= l_c_character_iterator.f_c_current_glyph;

            with l_c_glyph do
            begin
              Inc(l_running_x, l_glyph_fat);
              m_x:= l_running_x;
              Inc(l_running_x, m_width);

              // display(f_display_glyph);
              Dec(l_remaining_fat, l_glyph_fat);
              display(Format('  %-1s %3d + %3d => %3d , rem =%3d run=%3d',
                  [l_c_glyph.m_name, m_x- l_glyph_fat, l_glyph_fat, m_x, l_remaining_fat, l_running_x]));
            end; // with l_c_row_glyph
            Dec(l_glyph_count);

            l_c_character_iterator.goto_next_glyph;
          end; // while not l_c_character_iterator.f_is_done
        end; // recompute_glyph_x

      var l_fat: Integer;

      begin // format_line
        l_fat:= f_compute_total_fat;

        if l_fat> 0
          then recompute_glyph_x(l_fat);
      end; // justify_row

    begin // u_c_abstract_justification_strategy
    end. // u_c_abstract_justification_strategy
