// 003 u_c_control
// 18 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_control;
  interface
    uses Classes
        , ExtCtrls // PaintBox
        , Graphics
        , u_c_basic_object
        , u_c_pen_brush_stack
        ;

    const k_button_shadow= 2;
          k_scrollbar_cursor_height= 5;
    type t_on_mouse_down= Procedure(p_x, p_y: Integer) of Object;
         t_on_click= Procedure of Object;

         c_control_list= Class; // forward

         c_abstract_control= Class(c_basic_object)
                               m_c_control_list_ref: c_control_list;

                               // -- used as a trial for c_button
                               m_on_click: t_on_click;
                               m_on_mouse_down, m_on_mouse_move, m_on_mouse_up: t_on_mouse_down;

                               Constructor create_abstract_control(p_name: String;
                                   p_c_control_list_ref: c_control_list);
                               function f_display_control: String; Virtual;

                               function f_hit_test(p_x, p_y: Integer): Boolean; Virtual;
                               procedure draw_control; Virtual; Abstract;

                               procedure handle_control_mouse_down(p_x, p_y: Integer); Virtual;
                               procedure handle_control_mouse_move(p_x, p_y: Integer); Virtual;
                               procedure handle_control_mouse_up(p_x, p_y: Integer); Virtual;
                             end; // c_abstract_control

         c_control_list= class(c_basic_object)
                            m_c_paintbox_ref: tPaintbox;
                            m_c_canvas_ref: tCanvas;
                            m_c_save_canvas_stack: c_save_canvas_stack;

                            m_c_control_list: tStringList;

                            Constructor create_control_list(p_name: String;
                                 p_c_paintbox_ref: tPaintBox);

                            function f_control_count: Integer;
                            function f_c_abstract_control(p_control_index: Integer): c_abstract_control;
                            function f_index_of(p_control_name: String): Integer;
                            procedure add_control(p_control_name: String; p_c_abstract_control: c_abstract_control);

                            procedure display_control_list;

                            // -- mouse events
                            function f_c_find_mouse_control(p_x, p_y: Integer): c_abstract_control;

                            procedure handle_mouse_down(p_x, p_y: Integer);
                            procedure handle_mouse_move(p_x, p_y: Integer);
                            procedure handle_mouse_up(p_x, p_y: Integer);

                            procedure draw_control_list;

                            Destructor Destroy; Override;
                          end; // c_control_list

  implementation
    uses SysUtils
        , u_c_display
        ;

    // -- c_abstract_control

    Constructor c_abstract_control.create_abstract_control(p_name: String;
        p_c_control_list_ref: c_control_list);
      begin
        Inherited create_basic_object(p_name);
        m_c_control_list_ref:= p_c_control_list_ref;
        // -- add Yourself to the container
        m_c_control_list_ref.add_control(p_name, Self);
        // m_left:= p_left; m_top:= p_top; m_width:= p_width; m_height:= p_height;
      end; // create_abstract_control

    function c_abstract_control.f_display_control: String;
      begin
        Result:= m_name;
      end; // f_display_control

    function c_abstract_control.f_hit_test(p_x, p_y: Integer): Boolean;
      begin
        Result:= False;
      end; // f_hit_test

    procedure c_abstract_control.handle_control_mouse_down(p_x, p_y: Integer);
      begin
        // -- do nothing (== no event handler)
      end; // handle_control_mouse_down

    procedure c_abstract_control.handle_control_mouse_move(p_x, p_y: Integer);
      begin
      end; // handle_control_mouse_move

    procedure c_abstract_control.handle_control_mouse_up(p_x, p_y: Integer);
      begin
      end; // handle_control_mouse_up

    // -- c_control_list

    Constructor c_control_list.create_control_list(p_name: String;
        p_c_paintbox_ref: tPaintBox);
      begin
        Inherited create_basic_object(p_name);

        m_c_paintbox_ref:= p_c_paintbox_ref;
        m_c_canvas_ref:= m_c_paintbox_ref.Canvas;

        m_c_control_list:= tStringList.Create;
        m_c_save_canvas_stack:= c_save_canvas_stack.create_save_canvas_stack('save_pen_color', m_c_canvas_ref);
      end; // create_control_list

    function c_control_list.f_control_count: Integer;
      begin
        Result:= m_c_control_list.Count;
      end; // f_control_count

    function c_control_list.f_c_abstract_control(p_control_index: Integer): c_abstract_control;
      begin
        Result:= c_abstract_control(m_c_control_list.Objects[p_control_index]);
      end; //  f_c_abstract_control

    function c_control_list.f_index_of(p_control_name: String): Integer;
      begin
        Result:= m_c_control_list.IndexOf(p_control_name);
      end; // f_index_of

    procedure c_control_list.add_control(p_control_name: String; p_c_abstract_control: c_abstract_control);
      begin
        m_c_control_list.AddObject(p_control_name, p_c_abstract_control);
      end; // add_control

    procedure c_control_list.display_control_list;
      var l_control_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_control_count));

        for l_control_index:= 0 to f_control_count- 1 do
          display('  '+ f_c_abstract_control(l_control_index).f_display_control);
      end; // display_control_list

    function c_control_list.f_c_find_mouse_control(p_x, p_y: Integer): c_abstract_control;
      var l_control_index: Integer;
      begin
        Result:= Nil;

(*
        for l_control_index:= 0 to f_control_count- 1 do
          with f_c_abstract_control(l_control_index) do
            if f_hit_test(p_x, p_y)
              then begin
                  Result:= f_c_abstract_control(l_control_index);
                  Break;
                end;
*)
        // -- should loop over all, and only keep the top-most
        for l_control_index:= f_control_count- 1 downto 0 do
          with f_c_abstract_control(l_control_index) do
          begin
            if f_hit_test(p_x, p_y)
              then begin
                  Result:= f_c_abstract_control(l_control_index);
                  Break;
                end;
          end;
      end; // f_c_find_mouse_control

    procedure c_control_list.handle_mouse_down(p_x, p_y: Integer);
      var l_c_mouse_control: c_abstract_control;
      begin
        // display('> control_list.mouse_down');
        l_c_mouse_control:= f_c_find_mouse_control(p_x, p_y);

        if l_c_mouse_control<> Nil
          then begin
              display('control_down '+ l_c_mouse_control.f_display_control);
              l_c_mouse_control.handle_control_mouse_down(p_x, p_y);
            end;
        // display('< control_list.mouse_down');
      end; // handle_mouse_down

    procedure c_control_list.handle_mouse_move(p_x, p_y: Integer);
      var l_c_mouse_control: c_abstract_control;
      begin
        l_c_mouse_control:= f_c_find_mouse_control(p_x, p_y);

        if l_c_mouse_control<> Nil
          then begin
              // display('control_move '+ l_c_mouse_control.f_display_control);
              l_c_mouse_control.handle_control_mouse_move(p_x, p_y);
            end;
      end; // handle_mouse_move

    procedure c_control_list.handle_mouse_up(p_x, p_y: Integer);
      var l_c_mouse_control: c_abstract_control;
      begin
        // display('c_up');
        l_c_mouse_control:= f_c_find_mouse_control(p_x, p_y);
        if l_c_mouse_control<> Nil
          then l_c_mouse_control.handle_control_mouse_up(p_x, p_y);
      end; // handle_mouse_up

    procedure c_control_list.draw_control_list;
      var l_control_index: Integer;
      begin
        for l_control_index:= 0 to f_control_count- 1 do
        begin
          // display(IntToStr(l_control_index)+ ' '
          //     + f_c_abstract_control(l_control_index).f_display_control);
          f_c_abstract_control(l_control_index).draw_control;
        end;
(*
        for l_control_index:= 0 to f_control_count- 1 do
          f_c_abstract_control(l_control_index).draw_control;
*)
      end; // draw_control_list

    Destructor c_control_list.Destroy;
      var l_control_index: Integer;
      begin
        for l_control_index:= 0 to f_control_count- 1 do
          f_c_abstract_control(l_control_index).Free;
        m_c_control_list.Free;

        m_c_save_canvas_stack.Free;

        Inherited;
      end; // Destroy

    begin // u_c_control_list
    end. // u_c_control_list




