// 002 u_c_pen_brush_stack
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- save pen and brush usual attributes

(*$r+*)

unit u_c_pen_brush_stack;
  interface
    uses Classes, Graphics
        , u_c_basic_object
        ;

    type c_save_canvas= class(c_basic_object)
                          m_pen_color, m_pen_width: Integer;
                          m_pen_style: tPenStyle;
                          m_pen_mode: tPenMode;

                          m_brush_color: Integer;
                          m_brush_style: tBrushStyle;

                          m_font_height: Integer;

                          constructor create_save_canvas(p_name: String; p_c_canvas: tCanvas);
                          function f_c_self: c_save_canvas;
                          function f_display_save_canvas: String; Virtual;
                          procedure get_save_canvas(p_c_canvas_ref: tCanvas);
                        end; // c_save_canvas

         c_save_canvas_stack= Class(c_basic_object)
                                m_c_save_canvas_stack: tStringList;
                                m_c_canvas_ref: tCanvas;

                                Constructor create_save_canvas_stack(p_name: String;
                                    p_c_canvas_ref: tCanvas);

                                function f_save_canvas_count: Integer;
                                function f_c_save_canvas(p_save_canvas_index: Integer): c_save_canvas;

                                function f_is_empty: Boolean;
                                procedure push_save_canvas;
                                procedure pop_save_canvas;

                                procedure display_save_canvas_stack;

                                Destructor Destroy; Override;
                              end; // c_save_canvas_stack

  implementation
    uses SysUtils
        , u_c_display, u_graphic
        ;

    // -- c_save_canvas

    Constructor c_save_canvas.create_save_canvas(p_name: String; p_c_canvas: tCanvas);
      begin
        Inherited create_basic_object(p_name);

        with p_c_canvas do
        begin
          m_pen_color:= Pen.Color;
          m_pen_width:= Pen.Width;
          m_pen_style:= Pen.Style;
          m_pen_mode:= Pen.Mode;

          m_brush_color:= Brush.Color;
          m_brush_style:= Brush.Style;

          m_font_height:= Font.Height;
        end; // with p_c_canvas
      end; // create_integer

    procedure c_save_canvas.get_save_canvas(p_c_canvas_ref: tCanvas);
      begin
        with p_c_canvas_ref do
        begin
          Pen.Color:= m_pen_color;
          Pen.Width:= m_pen_width;
          Pen.Style:= m_pen_style;
          Pen.Mode:= m_pen_mode;

          Brush.Color:= m_brush_color;
          Brush.Style:= m_brush_style;

          Font.Height:= m_font_height;
        end; // with p_c_canvas
      end; // get_save_canvas

    function c_save_canvas.f_c_self: c_save_canvas;
      begin
        Result:= Self;
      end; // f_c_self

    function c_save_canvas.f_display_save_canvas: String;
      begin
        Result:= m_name
          + 'pen '+ f_color_name(m_pen_color)
          + 'br '+ f_color_name(m_brush_color);
      end; // f_display_save_canvas

    // -- c_save_canvas_stack

    Constructor c_save_canvas_stack.create_save_canvas_stack(p_name: String;
        p_c_canvas_ref: tCanvas);
      begin
        Inherited create_basic_object(p_name);

        m_c_save_canvas_stack:= tStringList.Create;
        m_c_canvas_ref:= p_c_canvas_ref;
      end; // create_save_canvas_line

    function c_save_canvas_stack.f_save_canvas_count: Integer;
      begin
        Result:= m_c_save_canvas_stack.Count;
      end; // f_save_canvas_count

    function c_save_canvas_stack.f_c_save_canvas(p_save_canvas_index: Integer): c_save_canvas;
      begin
        Result:= c_save_canvas(m_c_save_canvas_stack.Objects[p_save_canvas_index]);
      end; //  f_c_save_canvas

    function c_save_canvas_stack.f_is_empty: Boolean;
      begin
        Result:= f_save_canvas_count= 0;
      end; // f_is_empty

    procedure c_save_canvas_stack.push_save_canvas;
      var l_c_save_canvas: c_save_canvas;
      begin
        l_c_save_canvas:= c_save_canvas.create_save_canvas('', m_c_canvas_ref);
        m_c_save_canvas_stack.AddObject(l_c_save_canvas.m_name, l_c_save_canvas);
      end; // push_save_canvas

    procedure c_save_canvas_stack.pop_save_canvas;
      var l_c_save_canvas: c_save_canvas;
      begin
        if f_is_empty
          then
          else begin
              l_c_save_canvas:= f_c_save_canvas(f_save_canvas_count- 1);
              // -- reset the orignal tCanvas properties
              l_c_save_canvas.get_save_canvas(m_c_canvas_ref);
              l_c_save_canvas.Free;
              // -- delete the string and the cell in the tStringList
              m_c_save_canvas_stack.Delete(f_save_canvas_count- 1);
            end;
      end; // pop_save_canvas

    procedure c_save_canvas_stack.display_save_canvas_stack;
      var l_save_canvas_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_save_canvas_count));

        for l_save_canvas_index:= 0 to f_save_canvas_count- 1 do
          display(f_c_save_canvas(l_save_canvas_index).f_display_save_canvas);
      end; // display_save_canvas_stack

    Destructor c_save_canvas_stack.Destroy;
      var l_save_canvas_index: Integer;
      begin
        for l_save_canvas_index:= 0 to f_save_canvas_count- 1 do
          f_c_save_canvas(l_save_canvas_index).Free;
        m_c_save_canvas_stack.Free;

        Inherited;
      end; // Destroy

    begin // u_c_save_canvas
    end. // u_c_save_canvas
