// 002 u_c_caret
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_caret;
  interface
    uses
         u_c_basic_object
        , u_c_bridge_drawings
        , u_c_page_glyph
        , u_c_row_glyph
        ;

    const k_caret_default_height= 20;
          // -- the size of the serif
          k_caret_T= 2;

    type c_caret= Class(c_basic_object)
                    m_c_client_region_drawing_ref: c_client_region_drawing;

                    // -- where clicked with the mouse, then recomputed as
                    // --   the base line (low) point
                    m_screen_x, m_screen_y: Integer;
                    m_is_valid: Boolean;
                    // -- used to find where the cursor is positioned
                    m_c_root_page_glyph_ref: c_page_glyph;

                    m_caret_row_index, m_caret_element_index: Integer;
                    // -- depends on the row
                    m_caret_height: Integer;

                    constructor create_caret(p_name: String;
                        p_c_client_region_drawing_ref: c_client_region_drawing;
                        p_c_root_page_glyph_ref: c_page_glyph);
                    function f_display_caret: String;
                    procedure set_position(p_x, p_y: Integer);
                    procedure draw_caret;
                    function f_c_caret_row: c_row_glyph;
                  end; // c_caret

  implementation
    uses Graphics, SysUtils
        , u_c_display
        , u_c_iterator_kinds
        , u_c_glyph
        , u_c_glyph_kinds

        ;

    // -- c_caret

    constructor c_caret.create_caret(p_name: String;
        p_c_client_region_drawing_ref: c_client_region_drawing;
        p_c_root_page_glyph_ref: c_page_glyph);
      begin
        Inherited create_basic_object(p_name);

        m_c_client_region_drawing_ref:= p_c_client_region_drawing_ref;
        m_c_root_page_glyph_ref:= p_c_root_page_glyph_ref;
      end; // create_caret

    function c_caret.f_display_caret: String;
      begin
        Result:= Format('caret %3d %3d  r_ix =%3d  c_ix =%3d  H =%2d',
            [m_screen_x, m_screen_y, m_caret_row_index, m_caret_element_index, m_caret_height]);
      end; // f_display_caret

    procedure c_caret.set_position(p_x, p_y: Integer);

      procedure find_nearest_glyph;

        function f_c_find_row_position: c_row_glyph;
          var l_c_row_iterator: c_composite_iterator;
              // - avoid "holes" because of leadding
              l_running_top_y: Integer;
          begin
            display('> find_row mouse_y ='+ IntToStr(m_screen_y));

            // -- avoid warnings
            Result:= Nil;
            l_running_top_y:= 0;
            m_caret_row_index:= 0;
            m_caret_element_index:= 0;

            l_c_row_iterator:= c_composite_iterator.create_composite_iterator('row_iter',
                m_c_root_page_glyph_ref);
            l_c_row_iterator.goto_first_glyph;
(*
            while not l_c_row_iterator.f_is_done do
            begin
              display(l_c_row_iterator.f_c_current_glyph.f_display_glyph);
              l_c_row_iterator.goto_next_glyph;
            end;
*)
            l_c_row_iterator.goto_first_glyph;
            while not l_c_row_iterator.f_is_done do
            begin
              Result:= c_row_glyph(l_c_row_iterator.f_c_current_glyph);
              // Result:= c_l_c_row_iterator.f_c_current_glyph;
              if not (Result is c_row_glyph)
                then display_bug_stop('*** not_row');

              with Result do
              begin
                display(Format('row_top= %3d H= %3d  abs_y =%3d bot= %3d', [l_running_top_y,
                    m_height, f_absolute_y, f_absolute_y+ m_height])+ ' '+ m_name);
                if (m_screen_y> l_running_top_y) and (m_screen_y<= f_absolute_y+ m_height)
                  then Break;
                l_running_top_y:= f_absolute_y+ m_height;
              end; // with Result

              Inc(m_caret_row_index);
              l_c_row_iterator.goto_next_glyph;
            end; // while not l_c_row_iterator

            if l_c_row_iterator.f_is_done
              then begin
                  display('*** nil');
                  Result:= Nil;
                end;
            display('< row');
          end; // f_c_find_row_position

        function f_c_find_x_position(p_c_row_glyph: c_row_glyph): c_glyph;
          var l_c_glyph_element_iterator: c_composite_iterator;
          begin
            display(' find_element '+ IntToStr(m_screen_x));
            // -- avoid warnings
            Result:= Nil;
            l_c_glyph_element_iterator:= c_composite_iterator.create_composite_iterator('row', p_c_row_glyph);
            while not l_c_glyph_element_iterator.f_is_done do
            begin
              Result:= l_c_glyph_element_iterator.f_c_current_glyph;
              display('search_elem '+ Result.f_display_glyph
                  + ' X= '+ IntToStr(Result.f_absolute_x));
              if Result.f_absolute_x> m_screen_x
                then Break;
              l_c_glyph_element_iterator.goto_next_glyph;
              Inc(m_caret_element_index);
            end; // while not l_c_row_iterator

            if l_c_glyph_element_iterator.f_is_done
              then Result:= Nil;
          end; // f_c_find_x_position

        var l_c_row_glyph: c_row_glyph;
            l_c_glyph_element: c_glyph;

        begin // find_nearest_glyph
          if not m_c_root_page_glyph_ref.f_contains_glyph_leaves
            then Exit;

          display_line;  
          display('> find_nearest_glyph');

          l_c_row_glyph:= f_c_find_row_position;

          if l_c_row_glyph= Nil
            then begin
                display('no_row');
                m_caret_height:= k_caret_default_height;
                // -- place y after the end of the last line ?
                with m_c_root_page_glyph_ref do
                  l_c_row_glyph:= c_row_glyph(f_c_glyph(f_glyph_count- 1));
                display(l_c_row_glyph.f_display_glyph);
                m_screen_y:= l_c_row_glyph.f_absolute_y+ l_c_row_glyph.m_height
                    + k_caret_default_height;
                m_screen_x:= m_c_client_region_drawing_ref.m_drawing_left+ 2;
              end
            else begin
                display('ROW= '+ l_c_row_glyph.f_display_glyph);

                // -- recompute the caret y base point and height
                m_caret_height:= l_c_row_glyph.m_height;
                m_screen_y:= l_c_row_glyph.f_absolute_y+ l_c_row_glyph.m_height;
                display('recomp_y '+ f_display_caret);

                l_c_glyph_element:= f_c_find_x_position(l_c_row_glyph);
                if l_c_glyph_element= Nil
                  then begin
                      display('no_cur_ele');
                      // -- place the caret after the last glyph
                      l_c_glyph_element:= l_c_row_glyph.f_c_glyph(l_c_row_glyph.f_glyph_count- 1);
                      m_screen_x:= l_c_glyph_element.f_absolute_x+ l_c_glyph_element.m_width+ 2;
                    end
                  else begin
                      // display('CUR '+ l_c_glyph_element.m_name );
                      if not (l_c_glyph_element is c_leaf_glyph)
                        then display_bug_stop('not_element '+ l_c_glyph_element.ClassName);
                      display('CUR '+ l_c_glyph_element.f_display_glyph);
                      // -- recompute the caret x base point
                      m_screen_x:= l_c_glyph_element.f_absolute_x- 2;
                    end;
              end;

          display('< find_nearest_glyph '+ f_display_caret);
              // + Format('%3d %3d ', [m_caret_row_index, m_caret_element_index]));
        end; // find_nearest_glyph

      begin // set_position
        // -- delete previous
        if m_is_valid
          then draw_caret;
        m_screen_x:= p_x; m_screen_y:= p_y;

        find_nearest_glyph;

        m_is_valid:= True;
      end; // set_position

    procedure c_caret.draw_caret;
      begin
(*
        with m_c_paintbox_ref.Canvas do
        begin
          display(Format('%3d %3d', [m_screen_x, m_screen_y]));

          // Pen.Mode:= pmNot;
          Pen.Mode:= pmXor;
          Pen.Color:= clLime;

          MoveTo(m_screen_x, m_screen_y- m_caret_height+ 1);
          LineTo(m_screen_x, m_screen_y);

          MoveTo(m_screen_x- k_caret_T, m_screen_y- m_caret_height);
          LineTo(m_screen_x+ 1+ k_caret_T, m_screen_y- m_caret_height);

          MoveTo(m_screen_x- k_caret_T, m_screen_y);
          LineTo(m_screen_x+ 1+ k_caret_T, m_screen_y);

        end;
*)
        with m_c_client_region_drawing_ref do
        begin
          save_graphic_context;

          display(Format('%3d %3d', [m_screen_x, m_screen_y]));

          set_pen_mode(pmXor);
          set_pen_color(clLime);

          draw_line(m_screen_x, m_screen_y- m_caret_height+ 1,
              m_screen_x, m_screen_y);

          draw_line(m_screen_x- k_caret_T, m_screen_y- m_caret_height,
              m_screen_x+ 1+ k_caret_T, m_screen_y- m_caret_height);

          draw_line(m_screen_x- k_caret_T, m_screen_y,
              m_screen_x+ 1+ k_caret_T, m_screen_y);

          restore_graphic_context;
        end; // with m_c_client_region_drawing_ref
      end; // draw_caret

    function c_caret.f_c_caret_row: c_row_glyph;
      begin
        with m_c_root_page_glyph_ref do
          if m_caret_row_index< f_glyph_count
            then Result:= c_row_glyph(f_c_glyph(m_caret_row_index))
            else begin
                // -- insert another row
                Result:= f_c_append_row_glyph;
              end;
      end; // f_c_caret_row

    begin // u_c_yyy
    end. // u_c_yyy
