// 003 u_c_document
// 18 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- a document contains
// --  - the c_page_glyph
// --  - the selection rectangle

(*$r+*)

unit u_c_document;
  interface
    uses Classes, Windows, Graphics, ExtCtrls
        , u_c_basic_object
        , u_c_bridge_drawings
        , u_c_glyph
        , u_c_glyph_kinds
        , u_c_row_glyph
        , u_c_page_glyph
        , u_c_widget_factory

        , u_c_caret, u_c_selection

        , u_c_command
        ;

    const k_delta_font_height= 5;
          k_justification_width= 80;
          k_top_page_fat= 2;

    type t_insert_mode= (e_character, e_rectangle, e_bitmap);
         c_document= class(c_basic_object)
                       // -- uses the drawing primitive of the drawing surface
                       // -- this will shrink as frames and scrollbars are added
                       m_c_client_region_drawing: c_client_region_drawing;

                       // -- cannot use c_glyph since uses append
                       // -- cannot use c_page_glyph since might wants borders
                       // m_c_root_page_glyph: c_page_glyph;

                       m_c_root_page_glyph: c_page_glyph;

                       m_mouse_start_x, m_mouse_start_y: Integer;
                       // -- mouse move
                       m_current_x, m_current_y: Integer;
                       m_clicked, m_is_dragging: Boolean;

                       m_c_caret: c_caret;
                       m_c_selection_rectangle: c_selection_rectangle;

                       // -- initialized at creation, changed by the "+" and "-" buttons
                       m_font_height: Integer;

                       // -- mail boxes from the main form
                       m_justification_width: Integer;
                       m_bitmap_name: String;

                       m_c_command_stack: c_command_stack;
                       m_insert_mode: t_insert_mode;

                       Constructor create_document(p_name: String;
                           p_c_client_region_drawing: c_client_region_drawing);

                       procedure generate_initial_content;
                       procedure generate_random_content;

                       procedure display_document;
                       procedure display_glyph_hierarchy;
                       procedure display_text;

                       procedure draw_document;

                       procedure handle_mouse_down(p_x, p_y: Integer);
                       procedure handle_mouse_move(p_x, p_y: Integer);
                       procedure handle_mouse_up(p_x, p_y: Integer);

                       procedure clear_page;
                       procedure clear_character_area;
                       procedure draw_page_and_caret;

                       procedure draw_drag_rectangle;
                       procedure draw_selection_and_caret;

                         procedure _shrink_enclosed(p_decorator_count,
                             p_left_fat, p_top_fat, p_right_fat, p_bottom_fat: Integer);
                       procedure add_frame;
                       procedure add_scrollbar(p_c_widget_factory: c_abstract_widget_factory);

                         procedure change_font_height(p_new_font_height: Integer);
                       procedure increase_font_height_click;
                       procedure decrease_font_height_click;

                       // -- iterator
                       procedure list_glyphs_click;

                       // -- visitor
                       procedure count_glyphs_click;
                       procedure check_spelling_click;

                       // -- strategy
                       procedure justify_to_the_left_click;
                       procedure justify_both_sides_click;
                       procedure justify_to_the_right_click;
                       procedure justify_page_click;

                       // -- command
                         function f_c_create_leaf_glyph(p_character: Char;
                             p_c_row_glyph: c_row_glyph;
                             p_width, p_height: Integer): c_leaf_glyph;
                       function f_c_insert_leaf_glyph(p_character: Char): c_leaf_glyph;
                       function f_c_delete_character_glyph(p_row_index, p_element_index: Integer): c_character_glyph;

                       procedure do_delete_before_caret;

                       procedure do_insert_command(p_character: Char);
                       procedure undo_command_click;

                       procedure insert_letter_click;
                       procedure insert_rectangle_click;
                       procedure insert_bitmap_click;

                       Destructor Destroy; Override;
                     end; // c_document

  implementation
    uses SysUtils, Math
        , u_types_constants
        , u_c_display, u_strings, u_characters
        , u_graphic
        // -- for display
        , u_c_glyph_border_decorator_2
        , u_c_glyph_border_decorator_kinds_2

        , u_c_command_kinds
        ;

    // -- c_document

    Constructor c_document.create_document(p_name: String;
        p_c_client_region_drawing: c_client_region_drawing);
      begin
        Inherited create_basic_object(p_name);
        m_c_client_region_drawing:= p_c_client_region_drawing;

        with m_c_client_region_drawing do
          display('drawing '+ f_display_wh_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height));

        // -- no ancestor, max for clipping / hyphenation
        with m_c_client_region_drawing do
          m_c_root_page_glyph:= c_page_glyph.create_page_glyph('page',
              Nil,
              m_drawing_left, m_drawing_top+ k_top_page_fat, m_drawing_width, m_drawing_height- k_top_page_fat);

        with m_c_root_page_glyph do
          display('page '+ f_display_wh_rectangle(f_absolute_x, f_absolute_y, m_width_max, m_height_max));

        m_c_caret:= c_caret.create_caret('caret',
            m_c_client_region_drawing, m_c_root_page_glyph);
        m_c_selection_rectangle:= c_selection_rectangle.create_selection_rectangle('sel_rect',
            m_c_client_region_drawing, m_c_root_page_glyph);

        m_c_command_stack:= c_command_stack.create_command_stack('command_stack');
        // m_justification_width:= k_justification_width;
      end; // create_document

    procedure c_document.generate_initial_content;
      var l_row: Integer;

      procedure add_row_content(p_text: String);
        var l_c_row_glyph: c_row_glyph;
            l_c_character_glyph: c_character_glyph;
            l_character_index: Integer;
            l_character: Char;
            // l_character_count, l_space_index, l_mistake_index: Integer;
            // l_character_count: Integer;
        begin
          l_c_row_glyph:= c_row_glyph.create_row_glyph('row_'+ IntToStr(l_row),
              m_c_root_page_glyph);
          m_c_root_page_glyph.append_glyph(l_c_row_glyph);

          with l_c_row_glyph do
          begin
            // l_character_count:= 1+ Random(5);
            for l_character_index:= 1 to Length(p_text) do
            begin
              l_character:= p_text[l_character_index];

              with m_c_client_region_drawing do
                l_c_character_glyph:= c_character_glyph.create_character_glyph(l_character,
                    l_c_row_glyph, f_character_width(l_character), f_character_height(l_character));
              append_glyph(l_c_character_glyph);
            end; // for l_character_index
          end; // with l_c_row

          Inc(l_row);
        end; // add_row_content

      begin // generate_initial_content
        l_row:= 0;
        add_row_content('delphi is');
        // add_row_content('delphi');
        add_row_content('a wonderful');
        add_row_content('tool');

        // -- arrange the added glyphs
        with m_c_root_page_glyph do
        begin
          display_glyph;

          compute_placement;

          draw(m_c_client_region_drawing);
        end; // with g_c_document
      end; // generate_initial_content

    procedure c_document.generate_random_content;
      var l_row: Integer;
          l_c_row_glyph: c_row_glyph;
          l_c_character_glyph: c_character_glyph;
          l_character_index: Integer;
          l_character: Char;
          l_character_count, l_space_index, l_mistake_index: Integer;
      begin
        // -- add 3 rows with random characters
        for l_row:= 0 to 3 do
        begin
          l_c_row_glyph:= c_row_glyph.create_row_glyph('row_'+ IntToStr(l_row),
              m_c_root_page_glyph);
          m_c_root_page_glyph.append_glyph(l_c_row_glyph);

          with l_c_row_glyph do
          begin
            l_character_count:= 1+ Random(5);
            for l_character_index:= 0 to l_character_count do
            begin
              l_character:= Chr(Ord('a')+ Random(26));

              // -- insert spaces
              if (l_character_index> 0) and (l_character_index< l_character_count)
                then begin
                    l_space_index:= 1+ Random(5);
                    if l_space_index= l_character_index
                      then l_character:= ' '
                  end;

              // -- insert mis spelling
              if l_character<> ' '
                then begin
                    l_mistake_index:= 1+ Random(7);
                    if l_mistake_index= l_character_index
                      then l_character:= '%'
                  end;

              with m_c_client_region_drawing do
                l_c_character_glyph:= c_character_glyph.create_character_glyph(l_character,
                    l_c_row_glyph, f_character_width(l_character), f_character_height(l_character));
              append_glyph(l_c_character_glyph);
            end; // for l_character_index
          end; // with l_c_row
        end; // for l_row

        // -- arrange the added glyphs
        with m_c_root_page_glyph do
        begin
          display_glyph;

          compute_placement;

          draw(m_c_client_region_drawing);
        end; // with g_c_document
      end; // generate_random_content

    // -- display / draw

    procedure c_document.display_glyph_hierarchy;

      procedure display_hierarchy_recursive(p_c_composite_glyph: c_composite_glyph);
        var l_glyph_index: Integer;
        begin
          with p_c_composite_glyph do
          begin
            display(f_display_glyph);

            change_indentation(2);
            if p_c_composite_glyph is c_abstract_border_decorator
              then display_hierarchy_recursive(c_abstract_border_decorator(p_c_composite_glyph)._m_c_enclosed_glyph)
              else
                for l_glyph_index:= 0 to f_glyph_count- 1 do
                  if f_c_glyph(l_glyph_index) is c_leaf_glyph
                    then
                    else
                      if f_c_glyph(l_glyph_index) is c_composite_glyph
                        then display_hierarchy_recursive(c_composite_glyph(f_c_glyph(l_glyph_index)))
                        else ;

            change_indentation(-2);
          end; // with p_c_composite_glyph do
        end; // display_hierarchy_recursive

      begin // display_glyph_hierarchy
        display('> hierarchy');
        with m_c_client_region_drawing do
          display('client '+ f_display_wh_rectangle(m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height));

        display_hierarchy_recursive(m_c_root_page_glyph);
        display('< hierarchy');
      end; // display_glyph_hierarchy

    procedure c_document.display_document;

      procedure display_hierarchy_recursive(p_c_composite_glyph: c_composite_glyph);
        var l_glyph_index: Integer;
            l_glyph_string: String;
        begin
          with p_c_composite_glyph do
          begin
            display(f_display_glyph);

            change_indentation(2);
            if p_c_composite_glyph is c_abstract_border_decorator
              then display_hierarchy_recursive(c_abstract_border_decorator(p_c_composite_glyph)._m_c_enclosed_glyph)
              else begin
                  l_glyph_string:= '';
                  for l_glyph_index:= 0 to f_glyph_count- 1 do
                    if f_c_glyph(l_glyph_index) is c_leaf_glyph
                      then begin
                          if l_glyph_string<> ''
                            then l_glyph_string:= l_glyph_string + k_new_line
                              + f_spaces(f_indentation);
                          l_glyph_string:= l_glyph_string
                              + f_c_glyph(l_glyph_index).f_display_glyph;
                        end
                      else
                        if f_c_glyph(l_glyph_index) is c_composite_glyph
                          then display_hierarchy_recursive(c_composite_glyph(f_c_glyph(l_glyph_index)))
                          else ;
                  if l_glyph_string<> ''
                    then display(l_glyph_string);
                end;

            change_indentation(-2);
          end; // with p_c_composite_glyph do
        end; // display_hierarchy_recursive

      begin // display_document
        display('> display_document');
        display_hierarchy_recursive(m_c_root_page_glyph);
        display('< display_document');
      end; // display_document

    procedure c_document.display_text;
        // -- only view the text

      procedure display_text_recursive(p_c_composite_glyph: c_composite_glyph);
        var l_glyph_index: Integer;
            l_line: String;
        begin
          with p_c_composite_glyph do
          begin
            if p_c_composite_glyph is c_abstract_border_decorator
              then display_text_recursive(c_abstract_border_decorator(p_c_composite_glyph)._m_c_enclosed_glyph)
              else
                if p_c_composite_glyph is c_page_glyph
                  then
                    with c_page_glyph(p_c_composite_glyph) do
                      for l_glyph_index:= 0 to f_glyph_count- 1 do
                        display_text_recursive(c_row_glyph(f_c_glyph(l_glyph_index)))
                  else
                    if p_c_composite_glyph is c_row_glyph
                      then
                        with c_row_glyph(p_c_composite_glyph) do
                        begin
                          l_line:= '';
                          for l_glyph_index:= 0 to f_glyph_count- 1 do
                            l_line:= l_line+ f_c_glyph(l_glyph_index).m_name[1];
                          display(l_line);
                        end;
          end; // with p_c_composite_glyph do
        end; // display_text_recursive

      begin // display_text
        display('> display_text');
        display_text_recursive(m_c_root_page_glyph);
        display('< display_text');
      end; // display_text

    // -- drawing

    procedure c_document.clear_page;
        // -- this fills the client region, irrespective of all the frames and scrollbars
      begin
        with m_c_root_page_glyph, m_c_client_region_drawing do
          fill_rectangle(f_absolute_x, f_absolute_y, m_width_max, m_height_max, clWhite);
      end; // clear_page

    procedure c_document.clear_character_area;
        // -- this fills the "character" area
      begin
        with m_c_client_region_drawing do
          fill_rectangle(
             m_drawing_left, m_drawing_top, m_drawing_width, m_drawing_height,
             clWhite);
      end; // clear_character_area

    procedure c_document.draw_document;
      begin
        // -- for debug, display the client region
        m_c_client_region_drawing.draw_content;
        m_c_root_page_glyph.draw(m_c_client_region_drawing);
      end; // draw_document

    procedure c_document.draw_drag_rectangle;
      begin
        with m_c_client_region_drawing do
        begin
          save_graphic_context;

          set_pen_color(clAqua);
          set_pen_mode(pmXor);
          set_brush_style(bsClear);
          draw_rectangle(m_mouse_start_x, m_mouse_start_y,
              m_current_x- m_mouse_start_x, m_current_y- m_mouse_start_y);

          restore_graphic_context;
        end; // with m_c_client_region_drawing
      end; // draw_drag_rectangle

    procedure c_document.draw_selection_and_caret;
      begin
        // display('caret '+ f_display_TF(m_c_caret.m_is_valid)
        //   + '  sel '+ f_display_TF(m_c_selection_rectangle.m_is_valid));
        if m_c_selection_rectangle.m_is_valid
          then m_c_selection_rectangle.draw_selection_rectangle
          else
            if m_c_caret.m_is_valid
              then m_c_caret.draw_caret;
      end; // draw_selection_and_caret

    procedure c_document.draw_page_and_caret;
      begin
        draw_document;
        draw_selection_and_caret;
      end; // draw_page_and_caret

    // -- mouse

    procedure c_document.handle_mouse_down(p_x, p_y: Integer);
      begin
        // display('doc_mouse_down '+ Format('%3d %3d ', [p_x, p_y]));
        // Exit;

        // stop('doc_mouse_down');
        m_mouse_start_x:= p_x;
        m_mouse_start_y:= p_y;

        m_clicked:= True;

        // -- notify the page
      end; // handle_mouse_down

    procedure c_document.handle_mouse_move(p_x, p_y: Integer);
      begin
        // display('doc_mouse_move '+ Format('%3d %3d ', [p_x, p_y]));
        // Exit;

        if m_clicked and (Abs(p_x- m_mouse_start_x)> 5) and (Abs(p_y- m_mouse_start_y)> 5)
          then begin
              // -- erase previous
              if m_is_dragging
                then draw_drag_rectangle;
              m_is_dragging:= True;
              m_current_x:= p_x; m_current_y:= p_y;
              // -- display this one
              draw_drag_rectangle;
            end;
      end; // handle_mouse_down

    procedure c_document.handle_mouse_up(p_x, p_y: Integer);
      var l_selection_rectangle: tRect;
      begin
        // display('doc_mouse_up '+ Format('%3d %3d ', [p_x, p_y]));
        // Exit;

        if (Abs(p_x- m_mouse_start_x)> 3) and (Abs(p_y- m_mouse_start_y)> 3)
          then begin
              // -- selection
              // -- remove the drag rectangle
              draw_drag_rectangle;

              l_selection_rectangle:= Rect(m_mouse_start_x, m_mouse_start_y, p_x, p_y);

              with l_selection_rectangle do
                display('select_in_rect '+ Format('%3d %3d %3d %3d', [Left, Top, Right, Bottom]));

              // -- toggle m_is_selected of the elements within the rectangle
              m_c_root_page_glyph.select_in_rectangle(l_selection_rectangle);

              // -- initialize the selection rectangle
              m_c_selection_rectangle.set_selection_rectangle_position(l_selection_rectangle);
              if m_c_caret.m_is_valid
                then m_c_caret.draw_caret;

              // -- force green repainting of selected part
(* ds
              m_c_paintbox_ref.Repaint;
              m_c_root_page_glyph.draw(m_c_paintbox_ref);
*)
              m_c_caret.m_is_valid:= False;
              draw_selection_and_caret;
            end
          else begin
              // -- caret
              m_c_caret.set_position(p_x, p_y);

              // -- if rectangle is selected, delete the rectangle
              // --   and remove the green hilite
              if m_c_selection_rectangle.m_is_valid
                then begin
                    m_c_selection_rectangle.draw_selection_rectangle;
                  end;

              m_c_selection_rectangle.m_is_valid:= False;
              m_c_caret.draw_caret;
            end;

        m_clicked:= False;
        m_is_dragging:= False;
      end; // handle_mouse_up
{
    procedure c_document.display_selected_rectangle;
      var l_save_brush_type: tBrushStyle;
          l_save_pen_color: Integer;
      begin
(*
        with m_c_paintbox, Canvas do
        begin
          Repaint;
          m_c_root_page_glyph.draw(m_c_paintbox);

          l_save_brush_type:= Brush.Style;
          l_save_pen_color:= Pen.Color;

          Brush.Style:= bsClear;
          Pen.Color:= clRed;
          with m_selected_rectangle do
            // Rectangle(m_mouse_down_x, m_mouse_down_y, m_mouse_up_x, m_mouse_up_y);
            Rectangle(Left, Top, Right, Bottom);

          Pen.Color:= l_save_pen_color;
          Brush.Style:= l_save_brush_type;
        end; // with m_c_paintbox, Canvas
*)
      end; // display_selected_rectangle
}

    procedure c_document.change_font_height(p_new_font_height: Integer);
      begin
        display('> change_font_height '+ IntToStr(p_new_font_height));

        with m_c_client_region_drawing do
        begin
          save_graphic_context;

          m_font_height:= p_new_font_height;
          // -- change the canvas Font, which will be used to
          // --   recompute the text withs and heights
          set_font_height(m_font_height); // - StrToInt(edit1.Text);

          with m_c_root_page_glyph do
          begin
            change_selected_character_font(m_c_client_region_drawing);

            compute_placement;
            // ds draw(m_c_paintbox);
          end;

          // -- no longer needs the changed font height (will be
          // --  changed for each character at drawing time
          restore_graphic_context;

          // -- recompute the selection rectangle
          m_c_selection_rectangle.adjust_selection_to_glyphs;

          // draw_selection_and_caret;
          // clear_page;
          draw_page_and_caret;
        end; // with m_c_client_region_drawing
        display('< change_font_height');
      end; // change_font_height

    procedure c_document.increase_font_height_click;
      begin
        change_font_height(m_font_height+ k_delta_font_height);
      end; // increase_font_height_click

    procedure c_document.decrease_font_height_click;
      begin
        change_font_height(m_font_height- k_delta_font_height);
      end; // decrease_font_height

    // -- decorator

    var s_decorator_count: Integer= 0;

    procedure c_document._shrink_enclosed(p_decorator_count,
        p_left_fat, p_top_fat, p_right_fat, p_bottom_fat: Integer);

      procedure shrink_previous_recursive(p_level: Integer; p_c_composite_glyph: c_composite_glyph);
        // var l_c_abstract_border_decorator: c_abstract_border_decorator;
        begin
          display(f_spaces(p_level* 2)+ ' >'+ p_c_composite_glyph.m_name+ '< '+ IntToStr(p_level));
          if p_level>= 0
            then begin
                display('do_shrink');
                p_c_composite_glyph.shrink(p_left_fat, p_top_fat, - p_right_fat, - p_bottom_fat);

                if p_c_composite_glyph is c_abstract_border_decorator
                  then
                    with c_abstract_border_decorator(p_c_composite_glyph) do
                    begin
                      // display(f_spaces(s_border_count- p_level)+ 'shrink '+ m_name);
                      shrink_previous_recursive(p_level- 1, _m_c_enclosed_glyph);
                    end
                  else ; // display_bug_halt('not_c_border');
              end;
        end; // shrink_previous

      begin // _shrink_enclosed
        display('shrink');
        shrink_previous_recursive(p_decorator_count, m_c_root_page_glyph)
      end; // _shrink_enclosed

    procedure c_document.add_frame;
      const k_frame_width= 5;
      var l_c_frame_decorator: c_frame_decorator;
      begin
        display('> add_frame');

        clear_page;

        l_c_frame_decorator:= c_frame_decorator.create_frame_decorator(
            'frame_dec_'+ IntToStr(s_decorator_count),
            // -- the parent (for the first frame, NIL)
            m_c_root_page_glyph._m_c_parent_glyph,
            // -- the enclosed glyph
            m_c_root_page_glyph);

        // -- make room
        _shrink_enclosed(s_decorator_count, k_frame_width, k_frame_width, k_frame_width, k_frame_width);

        // -- also shrink the client region
        m_c_client_region_drawing.shrink_drawing(k_frame_width, k_frame_width, k_frame_width, k_frame_width);
        Inc(s_decorator_count);

        // -- the decorator becomes the root
        m_c_root_page_glyph:= l_c_frame_decorator;

        m_c_root_page_glyph.compute_placement;

        // clear_page;
        draw_document;

        display('< add_frame');
      end; // add_frame

    procedure c_document.add_scrollbar(p_c_widget_factory: c_abstract_widget_factory);
      const k_scroll_width= 12;
      var l_scrollbar_rectangle: t_wh_rectangle;
          l_c_scrollbar_drawing: c_scrollbar_drawing;
          l_c_scrollbar_decorator: c_scrollbar_decorator;
      begin
        display_line;
        display('> add_scroll');

        clear_page;
        // stop('');

        with m_c_root_page_glyph do
          l_scrollbar_rectangle:= f_wh_rectangle(f_absolute_x+ m_width_max- k_scroll_width- 3,
              f_absolute_y,
              k_scroll_width, m_height_max);

        display('root   '+ m_c_root_page_glyph.f_display_glyph);
        display('scroll '+ f_display_the_wh_rectangle(l_scrollbar_rectangle));

        with l_scrollbar_rectangle do
          l_c_scrollbar_drawing:= c_scrollbar_drawing.create_scrollbar_drawing(
              'scroll_draw_'+ IntToStr(s_decorator_count),
              m_c_client_region_drawing.m_c_canvas_ref,
              m_left, m_top, m_width, m_height);

        l_c_scrollbar_decorator:= c_scrollbar_decorator.create_scrollbar_decorator(
            'scroll_bar_dec_'+ IntToStr(s_decorator_count),
            // -- the parent (for the first frame, NIL)
            m_c_root_page_glyph._m_c_parent_glyph,
            // -- the enclosed glyph
            m_c_root_page_glyph,
            l_c_scrollbar_drawing);

        // -- this shrinks the c_glyph hierarchy
        _shrink_enclosed(s_decorator_count, 0, 0, k_scroll_width+ 6, 0);

        // -- also shrink the client region
        Dec(m_c_client_region_drawing.m_drawing_width, k_scroll_width+ 6);

        Inc(s_decorator_count);

        // -- the decorator becomes the root
        m_c_root_page_glyph:= l_c_scrollbar_decorator;

        m_c_root_page_glyph.compute_placement;

        draw_document;

        display('< add_scroll');
      end; // add_scrollbar

    procedure c_document.list_glyphs_click;
      begin
        m_c_root_page_glyph.list_glyphs(g_c_display.m_c_strings);
      end; // list_glyphs_click

    procedure c_document.count_glyphs_click;
      begin
        display('glyph_count '+ IntToStr(m_c_root_page_glyph.f_count_glyphs));
      end; // count_glyphs_click

    procedure c_document.check_spelling_click;
      var l_c_mistake_list: tStringList;
      begin
        l_c_mistake_list:= tStringList.Create;
        m_c_root_page_glyph.check_spelling(l_c_mistake_list);

        display_strings(l_c_mistake_list);
        l_c_mistake_list.Free;
      end; // check_spelling_click

    // -- strategy

    procedure c_document.justify_to_the_left_click;
      begin
        m_c_root_page_glyph.format_row_to_the_left(m_justification_width);
        draw_page_and_caret;
      end; // justify_to_the_left_click

    procedure c_document.justify_both_sides_click;
      begin
        m_c_root_page_glyph.format_row_both_sides(m_justification_width);
        draw_page_and_caret;
      end; // justify_both_sides_click

    procedure c_document.justify_to_the_right_click;
      begin
        m_c_root_page_glyph.format_row_to_the_right(m_justification_width);
        draw_page_and_caret;
      end; // justify_to_the_right_click

    procedure c_document.justify_page_click;
      begin
        m_c_root_page_glyph.format_page(m_justification_width);
        draw_page_and_caret;
      end; // justify_page_click

    // -- command

    function c_document.f_c_create_leaf_glyph(p_character: Char;
        p_c_row_glyph: c_row_glyph;
        p_width, p_height: Integer): c_leaf_glyph;
      begin
        // -- for the debugging, the letter are a..z, the rectangle A..Z and the
        // --   triangle 0..9
        case m_insert_mode of
          e_character :
            begin
              p_character:= f_lower_case(p_character);
              Result:= c_character_glyph.create_character_glyph(p_character,
                  p_c_row_glyph, p_width, p_height);
            end;
          e_rectangle :
            begin
              p_character:= Upcase(p_character);
              Result:= c_rectangle_glyph.create_rectangle_glyph(p_character,
                  p_c_row_glyph, p_width, p_height);
            end;
          e_bitmap :
            begin
              // -- for debugging, convert A..J into 0..9
              p_character:= Chr(Ord('0')+ (Ord(Upcase(p_character))- Ord('A')) mod 10);
              Result:= c_bitmap_glyph.create_bitmap_glyph(p_character,
                  p_c_row_glyph, p_width, p_height, m_bitmap_name);
            end;
        end; // case
      end; // f_c_create_leaf_glyph

    function c_document.f_c_insert_leaf_glyph(p_character: Char): c_leaf_glyph;
        // -- used by the c_insert_command
      var l_c_row_glyph: c_row_glyph;
          l_caret_element_index: Integer;
          l_next_caret_x, l_next_caret_y: Integer;
      begin
        display('> f_c_insert_leaf_glyph '+ p_character+ '<');
        with m_c_caret do
        begin
          l_c_row_glyph:= f_c_caret_row;
          l_caret_element_index:= m_caret_element_index;
        end; // with m_c_caret

        // -- create a new glyph
        with m_c_client_region_drawing do
          Result:= f_c_create_leaf_glyph(p_character, l_c_row_glyph,
              f_character_width(p_character), f_character_height(p_character));

        // -- insert in the composite structure
        l_c_row_glyph.insert_leaf_glyph(Result, l_caret_element_index);

        // -- recompute the placement
        m_c_root_page_glyph.compute_placement;

        // -- change the caret position
        with Result do
        begin
          // -- find the next position, "before" the next character
          l_next_caret_x:= f_absolute_x+ m_width- 2;
          l_next_caret_y:= f_absolute_y+ m_height- 2;
        end; // with Result

        display(m_c_caret.f_display_caret);
        display(Format('next %3d %3d ', [l_next_caret_x, l_next_caret_y]));
        m_c_caret.set_position(l_next_caret_x, l_next_caret_y);
        display(m_c_caret.f_display_caret);

        // -- redisplay, including the caret at the new position
        // -- for now, repaint the page because of remaining character
        // --   drooling
        clear_page;
        draw_page_and_caret;

        display('< f_c_insert_leaf_glyph');
      end; // f_c_insert_leaf_glyph

    function c_document.f_c_delete_character_glyph(p_row_index, p_element_index: Integer): c_character_glyph;
      begin
      end; // f_c_delete_character_glyph

    procedure c_document.do_insert_command(p_character: Char);
      var l_c_insert_command: c_insert_command;
      begin
        display('> do_insert_command');
        l_c_insert_command:= c_insert_command.create_insert_command(p_character, Self);

        l_c_insert_command.execute_command;
        // -- add the command
        m_c_command_stack.f_c_push_command(l_c_insert_command);
        display('< do_insert_command');
      end; // do_insert_command

    procedure c_document.do_delete_before_caret;
      begin
        // -- TODO
      end; // do_delete_before_caret

    procedure c_document.undo_command_click;
      begin
        display('> undo_command');
        m_c_command_stack.undo_command;
        display('< undo_command');
      end; // undo_command_click

    procedure c_document.insert_rectangle_click;
      begin
        m_insert_mode:= e_rectangle;
      end; // insert_rectangle_click

    procedure c_document.insert_bitmap_click;
      begin
        m_insert_mode:= e_bitmap;
      end; // insert_rectangle_click

    procedure c_document.insert_letter_click;
      begin
        m_insert_mode:= e_character;
      end; // insert_rectangle_click

    // -- free

    Destructor c_document.Destroy;
      begin
        m_c_command_stack.Free;

        Inherited;
      end; // Destroy

    begin
    end.
