// 002 u_c_selection
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_selection;
  interface
    uses Windows
        , u_c_basic_object
        , u_c_bridge_drawings
        , u_c_page_glyph
        ;

    const k_selection_fat= 2;
    type c_selection_rectangle= Class(c_basic_object)
                                  m_selected_rectangle: tRect;
                                  m_c_client_region_drawing_ref: c_client_region_drawing;
                                  m_is_valid: Boolean;

                                  // -- used to adjuste the rectangle to the selection
                                  m_c_root_page_glyph_ref: c_page_glyph;

                                  constructor create_selection_rectangle(p_name: String;
                                      p_c_client_region_drawing_ref: c_client_region_drawing;
                                      p_c_root_page_glyph_ref: c_page_glyph);
                                  procedure adjust_selection_to_glyphs;
                                  procedure set_selection_rectangle_position(p_rectangle: tRect);
                                  procedure draw_selection_rectangle;
                                end; // c_selection_rectangle

  implementation
    uses SysUtils, Math
         , Classes // Rect
         , Graphics
         , u_c_display, u_graphic
         , u_c_glyph // f_display_rectangle
         , u_c_iterator_kinds
         ;

    // -- c_selection_rectangle

    constructor c_selection_rectangle.create_selection_rectangle(p_name: String;
        p_c_client_region_drawing_ref: c_client_region_drawing;
        p_c_root_page_glyph_ref: c_page_glyph);
      begin
        Inherited create_basic_object(p_name);

        m_c_client_region_drawing_ref:= p_c_client_region_drawing_ref;
        m_c_root_page_glyph_ref:= p_c_root_page_glyph_ref;

        m_selected_rectangle:= Rect(0, 0, 0, 0);
      end; // create_selection_rectangle

    procedure c_selection_rectangle.adjust_selection_to_glyphs;
        // -- when increases the character size, should inflate the rectangle
      var l_enclosing_rectangle: tRect;  

      procedure compute_selection_enclosing_rectangle;
        var l_c_leaf_iterator: c_leaf_iterator;
        begin
          if not m_c_root_page_glyph_ref.f_contains_glyph_leaves
            then Exit;

          l_c_leaf_iterator:= c_leaf_iterator.create_leaf_iterator('page_iter', m_c_root_page_glyph_ref);

          with l_c_leaf_iterator do
          begin
            goto_first_glyph;

            while not f_is_done do
            begin
              with f_c_current_glyph do
              begin
                if m_is_selected
                  then begin
                      display('sel '+ f_display_glyph);
                      with l_enclosing_rectangle do
                      begin
                        if Left= 0
                          then begin
                              Left:= f_absolute_x;
                              Top:= f_absolute_y;
                            end
                          else begin
                              Left:= Min(Left, f_absolute_x);
                              Top:= Min(Top, f_absolute_y);
                            end;
                        Right:= Max(Right, f_absolute_x+ m_width);
                        Bottom:= Max(Bottom, f_absolute_y+ m_height);
                      end;
                    end
                  else display('not_sel '+ m_name);
              end;

              goto_next_glyph;
            end; // while not f_is_done
          end; // with l_c_leaf_iterator
        end; // compute_selection_enclosing_rectangle

      begin // adjust_selection_to_glyphs
        FillChar(l_enclosing_rectangle, SizeOf(tRect), 0);
        compute_selection_enclosing_rectangle;
        display(f_display_rectangle(l_enclosing_rectangle));

        // -- now inflate it a little bit
        with l_enclosing_rectangle do
        begin
          Dec(Left, k_selection_fat);
          Dec(Top, k_selection_fat);
          Inc(Right, 2* k_selection_fat);
          Inc(Bottom, 2* k_selection_fat);
        end;

        m_selected_rectangle:= l_enclosing_rectangle;
      end; // adjust_selection_to_glyphs

    procedure c_selection_rectangle.set_selection_rectangle_position(p_rectangle: tRect);
      begin
        display('> set_selection_rectangle');
        if m_is_valid
          then draw_selection_rectangle;

        m_selected_rectangle:= p_rectangle;
        m_is_valid:= True;
        adjust_selection_to_glyphs;
        display('< set_selection_rectangle');
      end; // set_selection_rectangle_position

    procedure c_selection_rectangle.draw_selection_rectangle;
      begin
        with m_c_client_region_drawing_ref do
        begin
          save_graphic_context;

          set_pen_color(clYellow);
          set_pen_mode(pmXor);
          set_brush_style(bsClear);
          with m_selected_rectangle do
            draw_rectangle(Left, Top, Right- Left- 2, Bottom- Top);

          restore_graphic_context;
        end; // with m_c_client_region_drawing_ref
      end; // draw_selection_rectangle

    begin // u_c_yyy
    end. // u_c_yyy
